/**
 * 
 */
package cs307.first;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.Iterator;

import cs307.first.BitVector.Builder;

/**
 * @author -
 *
 */
public class BitVectorBig_1 implements BitVector {
	
	/** Upper limit */
	private int upper;
	
	/** Lower limit */
	private int lower;
	
	/** Do we be strict and check the boundaries */
	private boolean strict;
	
	/** The builder for this BitVector */
	private Builder builder;
	
	/** Storage of the actual bits */
	private Hashtable<Integer, Boolean> vector;
	
	

	/**
	 * @param upper
	 * @param lower
	 * @param strict
	 * @param builder
	 */
	public BitVectorBig_1(int upper, int lower, boolean strict,
			Builder builder) {
		super();
		this.upper = upper;
		this.lower = lower;
		this.strict = strict;
		this.builder = builder;
		this.vector = new Hashtable<Integer, Boolean>();
	}
	
	

	/**
	 * @param upper
	 * @param lower
	 * @param strict
	 * @param builder
	 * @param vector
	 */
	public BitVectorBig_1(int upper, int lower, boolean strict,
			Builder builder, Hashtable vector) {
		super();
		this.upper = upper;
		this.lower = lower;
		this.strict = strict;
		this.builder = builder;
		this.vector = (Hashtable)vector.clone();
	}

	


	/**
	 * @return the vector
	 */
	public Hashtable<Integer, Boolean> getVector() {
		return vector;
	}



	/**
	 * @param vector the vector to set
	 */
	public void setVector(Hashtable<Integer, Boolean> vector) {
		this.vector = vector;
	}



	/* (non-Javadoc)
	 * @see cs307.first.BitVector#and(cs307.first.BitVector)
	 */
	@Override
	public void and(BitVector bs) {

		if (this.strict && !(bs instanceof BitVectorBig_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}
		
		Set<Integer> set = vector.keySet();
		java.util.Iterator<Integer> iter = set.iterator();
		
		while(iter.hasNext()) {
			Integer i = iter.next();
			if (this.test(i.intValue()) && bs.test(i.intValue())) {
				this.set(i.intValue());
			} else {
				this.unset(i.intValue());
			}
		}
		
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#builder()
	 */
	@Override
	public Builder builder() {
		
		return builder;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#copy()
	 */
	@Override
	public BitVector copy() {
		
		return new BitVectorBig_1(this.upper,
				this.lower,
				this.strict,
				this.builder,
				this.vector);
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#iterator()
	 */
	@Override
	public Iterator iterator() {
		
		return new Iterator_1(this, lower, upper, strict);
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#negate(int)
	 */
	@Override
	public boolean negate(int index) {
		
		boolean old;
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		old = vector.get(new Integer(index)).booleanValue();
		vector.put(new Integer(index), new Boolean(!old));
		
		return old;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#or(cs307.first.BitVector)
	 */
	@Override
	public void or(BitVector bs) {
		
		if (this.strict && !(bs instanceof BitVectorBig_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}

		
		Set<Integer> set = vector.keySet();
		Set<Integer> set2 = ((BitVectorBig_1) bs).getVector().keySet();
		java.util.Iterator<Integer> iter = set.iterator();
		
		while(iter.hasNext()) {
			Integer i = iter.next();
			if (this.test(i.intValue()) || bs.test(i.intValue())) {
				this.set(i.intValue());
			} else {
				this.unset(i.intValue());
			}
		}
		
		iter = set2.iterator();
		
		while(iter.hasNext()) {
			Integer i = iter.next();
			if (this.test(i.intValue()) || bs.test(i.intValue())) {
				this.set(i.intValue());
			} else {
				this.unset(i.intValue());
			}
		}
		
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#set(int)
	 */
	@Override
	public boolean set(int index) {

		boolean old;
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		Boolean bit = vector.get(new Integer(index));
		
		if (bit == null) {
			bit = new Boolean(false);
		}
		
		old = bit.booleanValue();
		vector.put(new Integer(index), new Boolean(true));
		
		return old;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#test(int)
	 */
	@Override
	public boolean test(int index) {

		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		Boolean bit = vector.get(new Integer(index));
		
		if (bit == null) {
			bit = new Boolean(false);
		}
		
		return bit.booleanValue();
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#unset(int)
	 */
	@Override
	public boolean unset(int index) {
		
		boolean old;
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		Boolean bit = vector.get(new Integer(index));
		
		if (bit == null) {
			bit = new Boolean(false);
		}
		
		old = bit.booleanValue();
		vector.put(new Integer(index), new Boolean(false));
		
		return old;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#xor(cs307.first.BitVector)
	 */
	@Override
	public void xor(BitVector bs) {
		
		if (this.strict && !(bs instanceof BitVectorBig_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}
		
		Set<Integer> set = vector.keySet();
		Set<Integer> set2 = ((BitVectorBig_1) bs).getVector().keySet();
		java.util.Iterator<Integer> iter = set.iterator();
		
		while(iter.hasNext()) {
			Integer i = iter.next();
			if (this.test(i.intValue()) ^ bs.test(i.intValue())) {
				this.set(i.intValue());
			} else {
				this.unset(i.intValue());
			}
		}
		
		iter = set2.iterator();
		
		while(iter.hasNext()) {
			Integer i = iter.next();
			if (!(set.contains(i))) {
				if (this.test(i.intValue()) ^ bs.test(i.intValue())) {
					this.set(i.intValue());
				} else {
					this.unset(i.intValue());
				}
			}
		}
		
	}

}
