/**
 * 
 */
package cs307.first;

import cs307.first.BitVector.Builder;

/**
 * @author -
 *
 */
public class BitVectorSmall_1 implements BitVector {
	
	/** Upper limit */
	private int upper;
	
	/** Lower limit */
	private int lower;
	
	/** Do we be strict and check the boundaries */
	private boolean strict;
	
	/** The builder for this BitVector */
	private Builder builder;
	
	/** Storage of the actual bits */
	private boolean[] vector;
	
	
	/** Create a new BitVector */
	public BitVectorSmall_1(int upper, int lower, boolean strict,
			Builder builder) {
		super();
		this.upper = upper;
		this.lower = lower;
		this.strict = strict;
		this.builder = builder;
		this.vector = new boolean[(upper - lower) + 1];
	}

	
	/**
	 * Used to generate a copy of a existing BitVector
	 * 
	 * @param upper
	 * @param lower
	 * @param strict
	 * @param builder
	 * @param vector
	 */
	public BitVectorSmall_1(int upper, int lower, boolean strict,
			Builder builder, boolean[] vector) {
		super();
		this.upper = upper;
		this.lower = lower;
		this.strict = strict;
		this.builder = builder;
		this.vector = vector;
	}


	/* (non-Javadoc)
	 * @see cs307.first.BitVector#and(cs307.first.BitVector)
	 */
	@Override
	public void and(BitVector bs) {
		
		if (this.strict && !(bs instanceof BitVectorSmall_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}
		
		Iterator bi = bs.iterator();
		
		boolean bit;
		for (int i = 0; i < vector.length; i++) {
			bit = bi.getNext();
			vector[i] = vector[i] && bit;
		}
		
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#builder()
	 */
	@Override
	public Builder builder() {
		
		return builder;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#copy()
	 */
	@Override
	public BitVector copy() {
		
		return new BitVectorSmall_1(this.upper, this.lower, this.strict, this.builder, this.vector);
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#iterator()
	 */
	@Override
	public Iterator iterator() {
		
		return new Iterator_1(this, lower, upper, strict);
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#negate(int)
	 */
	@Override
	public boolean negate(int index) {
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		this.vector[index] = !this.vector[index];
		
		return !this.vector[index];
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#or(cs307.first.BitVector)
	 */
	@Override
	public void or(BitVector bs) {
		
		if (this.strict && !(bs instanceof BitVectorSmall_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}
		
		Iterator bi = bs.iterator();
		
		boolean bit;
		for (int i = 0; i < vector.length; i++) {
			bit = bi.getNext();
			vector[i] = vector[i] || bit;
		}

	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#set(int)
	 */
	@Override
	public boolean set(int index) {

		boolean old;
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		old = this.vector[index];
		this.vector[index] = true;
		
		return old;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#test(int)
	 */
	@Override
	public boolean test(int index) {
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		return this.vector[index];
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#unset(int)
	 */
	@Override
	public boolean unset(int index) {
		
		boolean old;
		
		if (this.strict && (index > this.upper || index < this.lower)) {
			throw new ImplementationError("Out of active range bounds.");
		}
		
		old = this.vector[index];
		this.vector[index] = false;
		
		return old;
	}

	/* (non-Javadoc)
	 * @see cs307.first.BitVector#xor(cs307.first.BitVector)
	 */
	@Override
	public void xor(BitVector bs) {
		
		if (this.strict && !(bs instanceof BitVectorSmall_1)) {
			throw new ImplementationError("Invalid BitVector argument");
		}
		
		Iterator bi = bs.iterator();
		
		for (int i = 0; i < vector.length; i++) {
			vector[i] = vector[i] ^ bi.getNext();
		}

	}
	
	public int size() {
		
		return this.upper - this.lower;
	}

}
