package test.first.u1;

import static org.junit.Assert.*;

import cs307.first.*;
import cs307.first.BitVector.Iterator;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class BitVectorBigTest_1 {
	
	BitVector bv1;
	BitVector bv2;

	/**
	 * Main Setup for Unit Tests
	 * 
	 * Builds a <code>BitVector</code> for each of the tests.
	 * 
	 * @throws Exception
	 */
	@Before
	public void setUp() throws Exception {
		
		
		final Director director = new Director_1();
		
		final BitVector.Descriptor big = new BitVector.Descriptor(0, 1000000, true, false, true);
		
		final BitVector.Builder builder = (BitVector.Builder) director.makeBuilder(big);
		
		bv1 = builder.buildBitVector();
		bv2 = builder.buildBitVector();

	}

	
	/**
	 * Main Tear Down for Unit Tests
	 * 
	 * Destroys a <code>BitVector</code> after each of the tests.
	 * 
	 * @throws Exception
	 */
	@After
	public void tearDown() throws Exception {
		
		bv1 = null;
		bv2 = null;
		
	}
	
	/**
	 * Tests the SET, UNSET, and TEST operation on two <code>BitVector</code>'s
	 */
	@Test
	public void testSetUnsetTest() {
		
		bv1.set(0);
		bv1.set(1);
		bv1.set(42);
		bv1.set(100);
		bv1.set(1000);
		bv1.set(10000);
		
		assertEquals(bv1.test(0), true);
		assertEquals(bv1.test(1), true);
		assertEquals(bv1.test(42), true);
		assertEquals(bv1.test(100), true);
		assertEquals(bv1.test(1000), true);
		assertEquals(bv1.test(10000), true);
		
		assertEquals(bv1.test(3), false);
		assertEquals(bv1.test(4), false);
		assertEquals(bv1.test(53), false);
		assertEquals(bv1.test(1112), false);
		assertEquals(bv1.test(2312), false);

		bv1.unset(0);
		bv1.unset(1);
		bv1.unset(42);
		bv1.unset(100);
		bv1.unset(1000);
		bv1.unset(10000);
		
		assertEquals(bv1.test(0), false);
		assertEquals(bv1.test(1), false);
		assertEquals(bv1.test(42), false);
		assertEquals(bv1.test(100), false);
		assertEquals(bv1.test(1000), false);
		assertEquals(bv1.test(10000), false);
		
	}
	
	
	/**
	 * Tests the AND operation on two <code>BitVector</code>'s
	 */	
	@Test
	public void testAnd() {
		
		bv1.set(1);
		bv1.set(3);
		bv1.set(5);
		bv1.set(6);
		
		bv2.set(1);
		bv2.set(2);
		bv2.set(3);
		bv2.set(4);
		bv2.set(12);
		
		bv1.and(bv2);
		
		assertEquals(bv1.test(1), true);
		assertEquals(bv1.test(2), false);
		assertEquals(bv1.test(3), true);
		assertEquals(bv1.test(4), false);
		assertEquals(bv1.test(5), false);
		assertEquals(bv1.test(6), false);
		assertEquals(bv1.test(10), false);
		assertEquals(bv1.test(12), false);

	}
	
	/**
	 * Tests the OR operation on two <code>BitVector</code>'s
	 */
	@Test
	public void testOr() {
		
		bv1.set(1);
		bv1.set(3);
		bv1.set(5);
		bv1.set(6);
		
		bv2.set(1);
		bv2.set(2);
		bv2.set(3);
		bv2.set(4);
		bv2.set(12);
		
		bv1.or(bv2);
		
		assertEquals(bv1.test(1), true);
		assertEquals(bv1.test(2), true);
		assertEquals(bv1.test(3), true);
		assertEquals(bv1.test(4), true);
		assertEquals(bv1.test(5), true);
		assertEquals(bv1.test(6), true);
		assertEquals(bv1.test(10), false);
		assertEquals(bv1.test(12), true);

	}
	
	/**
	 * Tests the XOR operation on two <code>BitVector</code>'s
	 */
	@Test
	public void testXor() {
		
		bv1.set(1);
		bv1.set(3);
		bv1.set(5);
		bv1.set(6);
		
		bv2.set(1);
		bv2.set(2);
		bv2.set(3);
		bv2.set(4);
		bv2.set(12);
		
		bv1.xor(bv2);
		
		assertEquals(bv1.test(1), false);
		assertEquals(bv1.test(2), true);
		assertEquals(bv1.test(3), false);
		assertEquals(bv1.test(4), true);
		assertEquals(bv1.test(5), true);
		assertEquals(bv1.test(6), true);
		assertEquals(bv1.test(10), false);
		assertEquals(bv1.test(12), true);

	}
	
	/**
	 * Tests the <code>Iterator</code> for a <code>BitVector</code>
	 */
	@Test
	public void testIterator() {
		
		Iterator it = bv1.iterator();
		
		bv1.set(0);
		bv1.set(1);
		bv1.set(42);
		bv1.set(100);
		bv1.set(1000);
		bv1.set(10000);
		
		assertEquals(it.position(), 0);
		assertEquals(it.hasNext(), true);
		
		assertEquals(it.skipToSet(), 0);
		assertEquals(it.position(), 1);
		
		assertEquals(it.getNext(), true);
		assertEquals(it.getNext(), false);
		
		assertEquals(it.skipToSet(), 42);


		
	}
	
	

}
