/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cs307.first;

import threeohseven.BitVector;
import threeohseven.ImplementationError;

/**
 *
 * @author -
 */
public class BitVector_23_speed extends BitVector_23_base {

    private boolean[] bitVector = null;

    protected BitVector_23_speed(Builder b, boolean checked, int min, int max) {
        super(b, checked, min, max);
        bitVector = new boolean[max - min + 1];
    }

    /**
     * Returns a logically disjoint copy of the object.  The operation is
     * only valid if the two <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy() {
        BitVector_23_speed copy = new BitVector_23_speed(builder(), checked, min, max);
        System.arraycopy(bitVector, 0, copy.bitVector, 0, bitVector.length);
        return copy;
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    @Override
    public BitVector.Iterator iterator() {
        return new Iterator();
    }

    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index) {
        checkBounds(index);
        index -= min;
        boolean value = bitVector[index];
        bitVector[index] ^= true; //value xor true = negation.
        return value;
    }

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    @Override
    public void and(BitVector bs) {
        checkBounds(bs);
        BitVector.Iterator iterator = bs.iterator();
        while (iterator.position() < min) { //Move the iterator to the active range
            iterator.getNext();
        }
        while (iterator.hasNext() && iterator.position() <= max) {
            int pos = iterator.skipToUnset(); //We can only be affected by an AND if the value is false;
            if (iterator.hasNext() && pos <= max) {
                bitVector[pos - min] = iterator.getNext(); //Iterator.getNext() = false. F AND F = F, T AND F = F
            }
        }
    }

    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    @Override
    public void or(BitVector bs) {
        checkBounds(bs);
        BitVector.Iterator iterator = bs.iterator();
        while (iterator.position() < min) { //Move the iterator to the active range
            iterator.getNext();
        }
        while (iterator.hasNext() && iterator.position() <= max) {
            int pos = iterator.skipToSet();//We can only be affected if the value that we are OR ing with is True.
            if (iterator.hasNext() && pos <= max) {
                bitVector[pos - min] = iterator.getNext(); //Iterator.getNext() is true, F OR T = T, T OR T = T
            }
        }

    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    @Override
    public void xor(BitVector bs) {
        checkBounds(bs);
        BitVector.Iterator iterator = bs.iterator();

        while (iterator.position() < min) { //Move the iterator to the active range
            iterator.getNext();
        }
        while (iterator.hasNext() && iterator.position() <= max) {
            int pos = iterator.skipToSet();//The value can only change if we XOR with a True value. False XOR False = False, True XOR False = True
            if (iterator.hasNext() && pos <= max) {
                bitVector[iterator.position() - min] ^= iterator.getNext(); //getNext() = true.
            }
        }

    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index) {
        checkBounds(index);
        index -= min;
        boolean old = bitVector[index];
        bitVector[index] = true;
        return old;
    }

    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index) {
        checkBounds(index);
        index -= min;
        return bitVector[index];
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index) {
        checkBounds(index);
        index -= min;
        boolean old = bitVector[index];
        bitVector[index] = false;
        return old;
    }

    private class Iterator implements BitVector.Iterator {

        private int index = 0;

        /**
         * Returns the value for the current cursor, and advances the
         * cursor by one. When the cursor overflows the active range
         * (i.e. <code>hasNext()</code> is false) arbitrary values may be
         * returned. Implementations are not required to throw an exception.
         * @return the value of the next index in range.
         * @throws NoSuchElementError if hasNext() == false and vector is strict.
         **/
        public boolean getNext() {
            try {
                return bitVector[index++];
            } catch (ArrayIndexOutOfBoundsException e) {
                if (checked) {
                    throw new NoSuchElementError("No more elements");
                } else {
                    throw e;
                }
            }
        }

        /**
         * Are there more values in the active range?
         * @return false if the end of the range has been reached.
         **/
        public boolean hasNext() {
            return index < bitVector.length;
        }

        /**
         * The value of the cursor. This value may be larger than the
         * vector's upper bound. In which case <code>hasNext()</code> will
         * return false.
         * @return current position.
         **/
        public int position() {
            return index + min;
        }

        /**
         * Skips to the next set bit such that <code>getNext()</code> will
         * return true, if none is found the cursor is moved to the end of
         * the vector and <code>hasNext()</code> will return false from
         * thereon.
         * @return position of the first set bit, if there is none an
         * invalid value will be returned (ie a value larger than the upper
         * bound of the array).
         **/
        public int skipToSet() {
            while (hasNext()) {
                if (bitVector[index]) {
                    break;
                }
                index++;
            }
            return position();
        }

        /**
         * Skips to the next unset bit such that <code>getNext()</code>
         * will return false, if none is found the cursor is moved to the
         * end of the vector and <code>hasNext()</code> will return false
         * from thereon.
         * @return position of the first unset bit, if there is none an
         * invalid value will be returned (ie a value larger than the upper
         * bound of the array).
         **/
        public int skipToUnset() {
            while (hasNext()) {
                if (!bitVector[index]) {
                    break;
                }
                index++;
            }
            return position();
        }
    }
}
