
package cs307.first;

import threeohseven.BitVector;
import threeohseven.BitVector.Builder;
import threeohseven.BitVector.Iterator;
import threeohseven.ImplementationError;

/**
 * Contains the common implementation of builder methods for the 3 builders to use.
 * @author -
 */
public abstract class Builder_23_base implements BitVector.Builder {

    protected BitVector.Descriptor descriptor;

    public Builder_23_base(BitVector.Descriptor d) {
        this.descriptor = d;
    }

    public BitVector buildBitVector(BitVector val) {
        if (compatible(val.builder())) {
            return val.copy();
        } else {
            throw new ImplementationError("Builders are not compatible");
        }
    }

    public boolean compatible(Builder other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other instanceof Builder_23_base) {
            final Builder_23_base o = (Builder_23_base) other;
            if (o.descriptor.boundCheckRequired() == descriptor.boundCheckRequired()) {
                if (o.descriptor.getLowerBound() == descriptor.getLowerBound()) {
                    if (o.descriptor.getUpperBound() == descriptor.getUpperBound()) {
                        return true;
                    }
                }
            }
        } else {
            BitVector bv = other.buildBitVector();
            Iterator iter = bv.iterator();
            int min = iter.position();
            if (min != descriptor.getLowerBound()) {
                return false;
            }
            while (iter.hasNext()) {//Advance the iterator to the maximum position.
                iter.getNext();
            }
            int max = iter.position() - 1;
            if (max != descriptor.getUpperBound()) {
                return false;
            }
            try {
                iter.getNext(); //Will throw a checked error if the builer is checked, otherwise some exception (probably ArrayIndexOutOfBounds).
            } catch (ImplementationError e) {
                if (descriptor.boundCheckRequired()) {
                    return true;
                } else {
                    return false;
                }
            } catch (Exception e) {
                if (descriptor.boundCheckRequired()) {
                    return false;
                } else {
                    return true;
                }
            }
        }
        return false;
    }

    public Object build() {
        return buildBitVector();
    }

    /**
     *
     * @param o the object to test for equality.
     * @return Returns true if the descriptors are the same, false 
     * otherwise or if the object is not a Builder
     */
    @Override
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }

        if (o instanceof Builder_23_base) {
            final Builder_23_base other = (Builder_23_base) o;
            return other.descriptor.equals(descriptor);
        } else if (o instanceof Builder) {
            return ((Builder) o).compatible(this);
        }
        return false;
    }
}
