package test.first.u23;

import cs307.first.*;
import org.junit.Ignore;
import static org.junit.Assert.*;
import threeohseven.BitVector;
import threeohseven.BitVector.Builder;
import threeohseven.BitVector.Descriptor;
import threeohseven.BitVector.Iterator;
import threeohseven.ImplementationError;

/**
 * All BitVectors have the same testing requirements, so I have a test class per
 * BitVector implementation, and this provides the actual tests that are run.
 *
 *
 * @author -
 */
@Ignore //We do not want to run this as a test, as this is abstract and does not have a zero-arg constructor
public abstract class BitVector_23_baseTest {

    /**
     * This builds the regular builders that we use for the bulk of our tests.
     */
    private Builder baseBuilder;
    /**
     * The minimum active range baseBuilder will build
     */
    private final int baseBuilderMin;
    /**
     * The maximum active range baseBuilder will build.
     */
    private final int baseBuilderMax;
    /**
     * This builds BitVectors that are out of the active range of baseBuilder
     */
    private final Builder outOfRangeBuilder; //Builds BitVectors outside of the range of all of the builders.
    /**
     * The minimum active range outOfRangeBuilder will build.
     */
    private final int outOfRangeBuilderMin;
    /**
     * The maximum active range outOfRangeBuilder will build.
     */
    private final int outOfRangeBuilderMax;
    /**
     * This builder builds BitVectors that are completely inside of the active range of baseBuilder
     */
    private final Builder insideRangeBuilder;
    /**
     * The minimum active range insideRangeBuilder will build.
     */
    private final int insideRangeBuilderMin;
    /**
     * The maximum active range insideRangeBuilder will build.
     */
    private final int insideRangeBuilderMax;
    /**
     * This builder builds BitVectors that are only partly inside of the active range of baseBuilder
     */
    private final Builder partlyInsideRangeBuilder;
    /**
     * The minimum active range partlyInsideRangeBuilder will build.
     */
    private final int partlyInsideRangeBuilderMin;
    /**
     * The maximum active range partlyInsideRangeBuilder will build.
     */
    private final int partlyInsideRangeBuilderMax;

    public BitVector_23_baseTest(boolean space, boolean speed, boolean checked) {
        Director_23 director = new Director_23();

        baseBuilderMin = 1;
        baseBuilderMax = Short.MAX_VALUE - 1;
        Descriptor dd = new Descriptor(baseBuilderMin, baseBuilderMax, space, speed, checked);
        baseBuilder = director.makeBuilder(dd);

        outOfRangeBuilderMin = Short.MAX_VALUE;
        outOfRangeBuilderMax = Short.MAX_VALUE << 2;
        dd = new Descriptor(outOfRangeBuilderMin, outOfRangeBuilderMax, space, speed, checked);
        outOfRangeBuilder = director.makeBuilder(dd);

        insideRangeBuilderMin = 10;
        insideRangeBuilderMax = Short.MAX_VALUE >> 2;
        dd = new Descriptor(insideRangeBuilderMin, insideRangeBuilderMax, space, speed, checked);
        insideRangeBuilder = director.makeBuilder(dd);

        partlyInsideRangeBuilderMin = 10;
        partlyInsideRangeBuilderMax = Short.MAX_VALUE << 2;
        dd = new Descriptor(partlyInsideRangeBuilderMin, partlyInsideRangeBuilderMax, space, speed, checked);
        partlyInsideRangeBuilder = director.makeBuilder(dd);
    }

    /**
     * Test of and method, of class BitVector_23.
     */
    public void testAnd() {
        BitVector bs = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) { //Set all of the bits to one
            bs.set(i);
        }
        BitVector instance = baseBuilder.buildBitVector(); //This bitVector has all zeros
        bs.and(instance); //1 AND 0 should make all of bs to be zero.
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertFalse(bs.test(i));
        }
    }

    /**
     * Test of copy method, of class BitVector_23.
     */
    public void testCopy() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            instance.set(i); //Set the Bitvector so we know that it is not just returning a plain new copy.
        }
        BitVector expResult = instance;
        BitVector result = instance.copy();
        assertTrue(expResult.equals(result));
        result.negate(baseBuilderMin); //Test to make sure that the copied BitVector is not linked to the orignal bit vector
        assertTrue(result.test(baseBuilderMin) != instance.test(baseBuilderMin));
    }

    /**
     * Test of builder method, of class BitVector_23
     */
    public void testBuilder() {
        BitVector instance = baseBuilder.buildBitVector();
        assertEquals(instance.builder(), baseBuilder);
    }

    /**
     * Test of hasNext() of BitVector_23's iterator.
     */
    public void testHasNext() {

        BitVector instance = baseBuilder.buildBitVector();
        instance.set(baseBuilderMin);
        instance.set(baseBuilderMax);
        Iterator result = instance.iterator();
        int index = baseBuilderMin;
        while (result.hasNext()) {
            if (index == baseBuilderMin || index == baseBuilderMax) {
                boolean value = result.getNext();
                assertTrue(value);
            } else {
                result.getNext();
            }
            index++;
        }
        //The index will be 1 above the max when we come out of the loop as we have read the max index
        //So subtract one, and assert that the two maxes are equal to each other (aka it looped the appropriate # of times)
        assertEquals(index - 1, baseBuilderMax);
    }

    /**
     * Test of getNext() of BitVector_23's iterator.
     */
    public void testGetNext() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            if (i % 2 == 0) { //Set it every so often to true
                instance.set(i);
            }
        }
        Iterator result = instance.iterator();
        int index = baseBuilderMin;
        while (result.hasNext()) {
            if (index % 2 == 0) {
                assertTrue(result.getNext()); //If it was one of those we set to true, make sure its true
            } else {
                assertFalse(result.getNext()); //Otherwise make sure it is false.
            }
            index++;
        }
    }

    /**
     * Test of position() of BitVector_23's iterator.
     */
    public void testPosition() {

        BitVector instance = baseBuilder.buildBitVector();
        Iterator result = instance.iterator();
        int index = baseBuilderMin;
        /**
         * Loops through all values of the bitVector and asserts that
         * its index is the correct index.
         */
        while (result.hasNext()) {
            assertEquals(result.position(), index);
            result.getNext();
            index++;
        }
    }

    /**
     * Test of skipToSet() of BitVector_23's iterator.
     */
    public void testSkipToSet() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {

            if (i % 4 == 0) {
                instance.set(i); //Set some randomish variables to set so that we are sure it isn't just incrementing.
            }
        }
        Iterator result = instance.iterator();

        while (result.hasNext()) {
            result.skipToSet();
            if (result.hasNext()) {
                assertTrue(result.getNext());
            }
        }
    }

    /**
     * Test of skipToUnset() of BitVector_23's iterator.
     */
    public void testSkipToUnset() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            if (i % 4 != 0) {
                instance.set(i); //Same as above,except this time set all except when i is divisible by 4
            }
        }
        Iterator result = instance.iterator();

        while (result.hasNext()) {
            result.skipToUnset();
            if (result.hasNext()) {
                assertFalse(result.getNext());
            }
        }
    }

    /**
     * Test of negate method, of class BitVector_23.
     */
    public void testNegate() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            instance.negate(i); //This should set all to true
        }
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertTrue(instance.test(i));
        }
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            instance.negate(i); //This should set all to false, as before they were true.
        }
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertFalse(instance.test(i));
        }
    }

    /**
     * Test of or method, of class BitVector_23.
     */
    public void testOr() {
        BitVector bs = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            bs.set(i);
        }
        BitVector instance = baseBuilder.buildBitVector();
        instance.or(bs);
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertTrue(instance.test(i)); // 0 OR 1 = 1
        }
    }

    /**
     * Test of xor method, of class BitVector_23.
     */
    public void testXor() {
        BitVector bs = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            bs.set(i);
        }
        BitVector instance = baseBuilder.buildBitVector();
        instance.xor(bs);
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertTrue(instance.test(i)); //0 XOR 1 = 1
        }

        instance.xor(bs);
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertFalse(instance.test(i)); //1 XOR 1 = 0
        }
    }

    /**
     * Test of set method, of class BitVector_23.
     */
    public void testSet() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int index = baseBuilderMin; index <= baseBuilderMax; index++) {
            assertFalse(instance.set(index)); //Set should return false
            assertTrue(instance.test(index)); //And it should be true at the index.
        }
    }

    /**
     * Test of test method, of class BitVector_23.
     */
    public void testTest() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertFalse(instance.test(i)); //A BitVector initialized to false
        }
    }

    /**
     * Test of unset method, of class BitVector_23.
     */
    public void testUnset() {
        BitVector instance = baseBuilder.buildBitVector();
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            instance.set(i);
        }
        for (int i = baseBuilderMin; i <= baseBuilderMax; i++) {
            assertTrue(instance.unset(i)); //As the bitvector was set to all 1's above, this should return true
            assertFalse(instance.test(i)); //And since we just unset the index, test should return false.
        }
    }

    /**
     *
     * @param b the builder to get a bit vector from
     * @param min The minimum range it can go from
     * @param max The maximum range it can go to
     * @return A BitVector built by b with every index from min to max set to true
     */
    private BitVector getInstance(Builder b, int min, int max) {
        BitVector instance = b.buildBitVector();
        for (int i = min; i <= max; i++) {
            instance.set(i);
        }
        return instance;
    }

    /**
     * Tests to make sure that bv.and(instance) works properly
     * @param bv The bitVector that called and with instance as it's parameter
     * @param instance The bit vector that was returned by getInstance()
     */
    private void testAndFromInstance(BitVector bv, BitVector instance) {
        bv.and(instance);
        Iterator iter = bv.iterator();
        Iterator instanceIter = instance.iterator();
        while (iter.position() > instanceIter.position() && instanceIter.hasNext()) {
            instanceIter.getNext();
        }
        while (iter.position() < instanceIter.position() && iter.hasNext()) {
            iter.getNext();
        }
        while (iter.hasNext() && instanceIter.hasNext()) {
            assertFalse(iter.getNext());
            instanceIter.getNext();
        }
        resetBitVector(bv);
    }

    /**
     * Tests to make sure that bv.or(instance) works properly
     * @param bv The bitVector that called or with instance as it's parameter
     * @param instance The bit vector that was returned by getInstance()
     */
    private void testOrFromInstance(BitVector bv, BitVector instance) {
        bv.or(instance);
        Iterator iter = bv.iterator();
        Iterator instanceIter = instance.iterator();
        while (iter.position() > instanceIter.position() && instanceIter.hasNext()) {
            instanceIter.getNext();
        }
        while (iter.position() < instanceIter.position() && iter.hasNext()) {
            iter.getNext();
        }
        while (iter.hasNext() && instanceIter.hasNext()) {
            assertTrue(iter.getNext());
            instanceIter.getNext();
        }
        resetBitVector(bv);
    }

    /**
     * Tests to make sure that bv.xor(instance) works properly
     * @param bv The bitVector that called xor with instance as it's parameter
     * @param instance The bit vector that was returned by getInstance()
     */
    private void testXorFromInstance(BitVector bv, BitVector instance) {
        bv.xor(instance);
        Iterator iter = bv.iterator();
        Iterator instanceIter = instance.iterator();
        while (iter.position() > instanceIter.position() && instanceIter.hasNext()) {
            instanceIter.getNext();
        }
        while (iter.position() < instanceIter.position() && iter.hasNext()) {
            iter.getNext();
        }
        while (iter.hasNext() && instanceIter.hasNext()) {
            assertTrue(iter.getNext());
            instanceIter.getNext();
        }
        resetBitVector(bv);
    }

    /**
     * Resets a Bit Vector to all zeros.
     * @param bv The BitVector to reset.
     *
     */
    private void resetBitVector(BitVector bv) {
        Iterator iter = bv.iterator();
        while (iter.hasNext()) {
            bv.unset(iter.position());
            iter.getNext();
        }
    }

    /**
     * Tests all of the different combinations that bit vectors can be (aka overlaping, inside, outside)
     */
    public void testBounds() {
        BitVector bv1 = baseBuilder.buildBitVector();
        //Test the same range.
        try {
            BitVector instance = getInstance(baseBuilder, baseBuilderMin, baseBuilderMax);

            testAndFromInstance(bv1, instance);
            testOrFromInstance(bv1, instance);
            testXorFromInstance(bv1, instance);
        } catch (ImplementationError e) {
            e.printStackTrace();
            fail("Failed with the same bit vector");
        }
        //Test with the range outside of the others.
        boolean andPassed = true;
        boolean orPassed = true;
        boolean xorPassed = true;
        try {
            BitVector instance = getInstance(outOfRangeBuilder, outOfRangeBuilderMin, outOfRangeBuilderMax);

            testAndFromInstance(bv1, instance);
            andPassed = false;
            testOrFromInstance(bv1, instance);
            orPassed = false;
            testXorFromInstance(bv1, instance);
            xorPassed = false;
        } catch (ImplementationError e) {

        } catch (Exception e) { //Catch any exception caused by doing an out of range and/or/xor
        }
        //We do this so that we know that and, or and xor all pass and not just one of the tests.
        if(!(andPassed && orPassed && xorPassed)) {
            fail("Not all tests passed for out of range.\n AND: "+andPassed+" OR: "+orPassed+" XOR: "+xorPassed);
        }
        //Test with the range inside of the other
        try {
            BitVector instance = getInstance(insideRangeBuilder, insideRangeBuilderMin, insideRangeBuilderMax);
            testAndFromInstance(bv1, instance);
            testOrFromInstance(bv1, instance);
            testXorFromInstance(bv1, instance);
        } catch (ImplementationError e) {
            e.printStackTrace();
            fail("Failed with a bit vector inside of the other's range");
        }
        //Test with the range starting inside and going outside of the others.
        try {
            BitVector instance = getInstance(partlyInsideRangeBuilder, partlyInsideRangeBuilderMin, partlyInsideRangeBuilderMax);
            testAndFromInstance(bv1, instance);
            testOrFromInstance(bv1, instance);
            testXorFromInstance(bv1, instance);
        } catch (ImplementationError e) {
            e.printStackTrace();
            fail("Failed with a bit vector overlaping of the other's range");
        }
    }
}
