package cs307.first;

import threeohseven.BitVector;
import threeohseven.ImplementationError;

/**
 * This BitVector implementation is optimized for small bit vectors where
 * the test, or and and functions are speed critical. It stores the vector
 * as an array of booleans.
 * 
 * @author -
 */
public class BitVectorS_46 implements BitVector {
	private boolean _checked; //true if bound checking is enforced
	public int _min, _max; //min and max index of the bit vector
	public boolean[] _bv; //stores the vector
	public Builder builder; //builder used to make this bit vector

	public BitVectorS_46() {
		super();
	}

	/**
	 * The constructor initializes the boolean array to min-max+1 false bits.
	 * 
	 * @param min vector index
	 * @param max vector index
	 * @param checked (true of bound checking is enforced)
	 */
	public BitVectorS_46(int min, int max, boolean checked) {
		_max = max;
		_min = min;
		_bv = new boolean[max-min+1];
		_checked = checked;
		for (int i=min; i<=max; i++) {
			_bv[i-min] = false;
		}
	}

	/**
	 * Makes and returns a string representing the bit vector.
	 * @return string
	 */
	@Override
	public String toString() {
		String str = "";
		for (int i=0; i<_max-_min+1; i++) {
			str += _bv[i] ? "1" : "0";
		}
		return str;
	}

	/**
	 * Given a BitVector bs, determines whether bs is compatible with 
	 * this. If so, it performs an AND operation on each pair of bits
	 * and stores the result in this BitVector.
	 * @param BitVector bs
	 */
	@Override
	public void and(BitVector bs) {
		BitVectorS_46 bvs = (BitVectorS_46) bs;
		if (bvs._min == _min && bvs._max == _max) {
			for (int i=0; i<_max-_min+1; i++) {
				this._bv[i] = this._bv[i] && bvs._bv[i];
			}
		}
		else {
			throw new ImplementationError("incompatible BitVectors");
		}
	}

	/**
	 * Returns a copy of this BitVector's builder.
	 * @return BuilderS_mkorb
	 */
	@Override
	public Builder builder() {
		return builder;
	}

	/**
	 * Returns a copy of this BitVector by using the builder to create a new one.
	 * @return BitVectorS_mkorb
	 */
	@Override
	public BitVector copy() {
		return builder.buildBitVector(this);
	}

	/**
	 * Returns an Iterator that can be used to access this BitVector
	 * @return Iterator
	 */
	@Override
	public Iterator iterator() {
		return new IteratorS_46(this);
	}

	/**
	 * Negates the bit located at index and returns the old value.
	 * @return boolean
	 */
	@Override
	public boolean negate(int index) {
		if (index > _max || index < _min) {
			if (_checked) {
				throw new ImplementationError("Out of bounds");
			}
			else {
				return false;
			}
		}
		_bv[index-_min] = !_bv[index-_min];
		return !_bv[index-_min];
	}

	/**
	 * Given a BitVector bs, determines whether bs is compatible with 
	 * this. If so, it performs an OR operation on each pair of bits
	 * and stores the result in this BitVector.
	 * @param BitVector bs
	 */
	@Override
	public void or(BitVector bs) {
		BitVectorS_46 bvs = (BitVectorS_46) bs;
		if (bvs._min == _min && bvs._max == _max) {
			for (int i=0; i<_max-_min+1; i++) {
				this._bv[i] = this._bv[i] || bvs._bv[i];
			}
		}
		else {
			throw new ImplementationError("incompatible BitVectors");
		}
	}

	/**
	 * Sets the bit at index and returns the old value.
	 * @return boolean
	 */
	@Override
	public boolean set(int index) {
		if (index > _max || index < _min) {
			if (_checked) {
				throw new ImplementationError("Out of bounds");
			}
			else {
				return false;
			}
		}
		boolean oldval = _bv[index-_min];
		_bv[index-_min] = true;
		return oldval;
	}

	/**
	 * Returns the value of the bit at index.
	 * @return boolean
	 */
	@Override
	public boolean test(int index) {
		if (index > _max || index < _min) {
			if (_checked) {
				throw new ImplementationError("Out of bounds");
			}
			else {
				return false;
			}
		}
		return _bv[index-_min];
	}

	/**
	 * Unsets the bit at index and returns the old value.
	 * @return boolean
	 */
	@Override
	public boolean unset(int index) {
		if (index > _max || index < _min) {
			if (_checked) {
				throw new ImplementationError("Out of bounds");
			}
			else {
				return false;
			}
		}
		boolean oldval = _bv[index-_min];
		_bv[index-_min] = false;
		return oldval;
	}

	/**
	 * Given a BitVector bs, determines whether bs is compatible with 
	 * this. If so, it performs an XOR operation on each pair of bits
	 * and stores the result in this BitVector.
	 * @param BitVector bs
	 */
	@Override
	public void xor(BitVector bs) {
		BitVectorS_46 bvs = (BitVectorS_46) bs;
		if (bvs._min == _min && bvs._max == _max) {
			for (int i=0; i<_max-_min+1; i++) {
				this._bv[i] = this._bv[i] ^ bvs._bv[i];
			}
		}
		else {
			throw new ImplementationError("incompatible BitVectors");
		}
	}
}
