/**
 * The BitVector interface for 307-PA1
 **/
package cs307.first; 

import cs307.first.Vector_51;
import cs307.threeohseven.*;
//import first.*;

/**
 * A BitVector maps values from an active range to booleans. The bit
 * vector's active range is denoted [low, high] such that low and high are
 * int value and low <= high. Any index within that range is valid.<p>
 *
 * The interface defines methods testing and setting individual positions
 * in the set's active range (<code>set, test, unset, negate</code>), as
 * well as some binary logical operations (<code>or, xor, and,
 * etc.</code>).<p>
 *
 * Each vector instance has a unique <code>Builder</code>, the builder is
 * the object that created this vector. Builders are immutable values thus
 * two vectors created by the same builder are guaranteed to be compatible.
 * More precisely, we say that two <code>BitVector</code> objects are
 * <em>compatible</em> if their builders are
 * <code>Buidler.compatible()</code>.<p>
 * 
 * Binary operations on <code>BitVector</code> are valid if and only if the
 * two BitVectors are compatible. Thus it is not valid to attempt to
 * <code>or</code> bit vector with different active ranges.
 *
 * With regards to error checking, an implementation may either be
 * <em>strict</em> or <em>relaxed</em>. A strict implementation must check
 * all operations and throw exception when an invalid operation is
 * attempted. A relaxed implementation may let some errors go
 * unreported. The motivation for the relaxed mode is to allow for some
 * common implementation idioms (for speed).  The relaxed mode should not
 * throw any undeclared exceptions.
 *
 * @author -
 * @version 1.1.0
 **/
public class BitVector_51 implements BitVector{
    private Vector_51 v;  
    private int min;
    private int max;
    private boolean strict;

    /** Default constructor, sets size of vector to 20 
     * 
     */
    public BitVector_51(){
    	v = new Vector_51(20);
    	for(int i=0;i<20;i++)
    	{
    		((Vector_51)v).setElementAt(new Boolean(false),i);
    	}
    }

    /** Constructor takes integer parameter which sets the size of the vector
     * 
     * @param size size of bit vector
     */
    public BitVector_51(int size){
    	v = new Vector_51(size);
    	for(int i=0;i<size;i++)
    	{
    		((Vector_51)v).setElementAt(new Boolean(false),i);
    	}
    }
    /** Constructor takes parameters for min and max bits
     * 
     * @param min_ minimum bit of bit vector
     * @param max_ maximum bit of bit vector
     */
    public BitVector_51(int min_, int max_){
    	int size = max_ - min_;
    	v = new Vector_51(size);
    	min = min_;
    	max = max_;
    	for(int i=0;i<size;i++)
    	{
    		((Vector_51)v).setElementAt(new Boolean(false),i);
    	}
    }
    
    /** Constructor takes parameters for size of vector, min, and max bits
     * 
     * @param size size of bit vector
     * @param min_ minimum bit of bit vector
     * @param max_ maximum bit of bit vector
     */
    public BitVector_51(int size, int min_, int max_){
    	v = new Vector_51(size);
    	min = min_;
    	max = max_;
    	for(int i=0;i<size;i++)
    	{
    		((Vector_51)v).setElementAt(new Boolean(false),i);
    	}
    }
    
    /** Constructor takes descriptor as a parameter
     * 
     * @param d descriptor
     */
    public BitVector_51(Descriptor_51 d){
     	this.min = d.getLowerBound();
    	this.max = d.getUpperBound();
    	this.strict = d.boundCheckRequired();
    	int size = max - min;
    	this.v = new Vector_51(size);
    	for(int i=0;i<v.size();i++)
    	{
    		this.v.setElementAt(new Boolean(false),i);
    	}
    }

    /** 
     * @return minimum of bit vector
     */
    public int getMin()
    {
    	return this.min;
    }
    
    /**
     * @return maximum of bit vector
     */
    public int getMax()
    {
    	return this.max;
    }
   
    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return value at <code>index</code> 
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index)
    {
    	if ((index < this.getMin()) || (index > this.getMax()) )
    	{
    		if(this.strict)
    			throw new ImplementationError("Implementation Error");
    		else
    			return false;
    	}

    	if( this.v.elementAt(index).toString()=="true" )
    		return true;

    	return false;
    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index)
    {
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
    	
    	if ((index < this.getMin()) || (index > this.getMax()) )		
    	{
    		if(this.strict)
				throw new ImplementationError("Implementation Error");
    		else
    			return false;
    	}
    	if( this.v.elementAt(index).equals(f))
    	{  
    		//System.out.println("in set, element was false");
    		this.v.setElementAt(t,index);
    		//System.out.println("element at "+index+" is "+this.v.elementAt(index));
    		return false;
    	}
    	System.out.println("element was true");
    	return true;
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index)
    {
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
    	
    	if ((index < this.getMin()) || (index > this.getMax()) )
    	{
    		if(this.strict)
				throw new ImplementationError("Implementation Error");
    		else
    			return false;
    	}
	
    	if( this.v.elementAt(index).equals(t) )
    	{  
    		this.v.setElementAt(f,index);
    		return true;
    	}
    	return false;
    }

    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index)
    {
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
    	
    	if ((index < this.getMin()) || (index > this.getMax()) )
    	{
    		if(this.strict)
				throw new ImplementationError("Implementation Error");
    		else
    			return false;
    	}
	
    	if( this.v.elementAt(index).equals(t) )
    	{  
    		this.v.setElementAt(f,index);
    		return true;
    	}
	
    	if( this.v.elementAt(index).equals(f))
    	{  
    		this.v.setElementAt(t,index);
    		return false;
    	}

    	return false;
    }

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void and(BitVector bs)
    {}
    public void and(BitVector_51 bs)
    {
    	int i;
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
	
    	if((this.getMin() != bs.getMin()) || (this.getMax() != bs.getMax()))
    	{
    		if(this.strict)
				throw new ImplementationError("Implementation Error");
    	}

    	for(i=this.getMin();i<this.getMax();i++)
    	{
    		if( this.v.elementAt(i).equals(t) && bs.v.elementAt(i).equals(t) )
    		{
    			this.v.setElementAt(t,i);
    		}
    		else
    			this.v.setElementAt(f,i);
    	}
    }

    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void or(BitVector bs)
    {    }
    public void or(BitVector_51 bs)
    {
    	int i;
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
	
	if((this.getMin() != bs.getMin()) || (this.getMax() != bs.getMax()))
	{ 
    	if(this.strict)
			throw new ImplementationError("Implementation Error");
	}
	for(i=this.getMin();i<this.getMax();i++)
	{
		if( this.v.elementAt(i).equals(t) || bs.v.elementAt(i).equals(t) )
		{
			this.v.setElementAt(t,i);
		}
		else
			this.v.setElementAt(f,i);
	}
    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void xor(BitVector bs)
    {}
    public void xor(BitVector_51 bs)
    {
    	int i;
    	Boolean t = new Boolean(true);
    	Boolean f = new Boolean(false);
    	if((this.getMin() != bs.getMin()) || (this.getMax() != bs.getMax()))
    	{
    		if(this.strict)
				throw new ImplementationError("Implementation Error");
    	}
    	for(i=this.getMin();i<this.getMax();i++)
    	{
    		if( this.v.elementAt(i).equals(t) && bs.v.elementAt(i).equals(f) )
		{
			this.v.setElementAt(t,i);
		}
		else if( this.v.elementAt(i).equals(f) && bs.v.elementAt(i).equals(t) )
		{
			this.v.setElementAt(t,i);
		}
		else
			this.v.setElementAt(f,i);
	}
    }

    /**
     * Returns a logically disjoint copy of the object.  The operation is
     * only valid if the two <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy()
    {
    	//TODO:  all
    	BitVector_51 i = new BitVector_51(this.v.size(), this.getMin(), this.getMax());
    	//TODO: copy vector
    	return (BitVector_51)i;
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    public Iterator iterator()
    {
    	//TODO:  all
    	Iterator_51 i = new Iterator_51();
    	return (Iterator)i;
    }

    /**
     * Returns the builder that created this <code>BitVector</code>.
     **/
    public Builder builder()
    {
    	//TODO:  all
    	Builder_51 i = new Builder_51();
    	return (Builder)i;
    }

    /**
     * Return a representation of this <code>BitVector</code> as a
     * string. The format is implementation specific.
     **/
    public String toString()
    {
    	//Boolean f = new Boolean(false);
    	Boolean t = new Boolean(true);
    	String s="";
		for(int i=0;i<this.v.size();i++)
		{
			if(this.v.elementAt(i).equals(t))
			{
				s+="1 ";
			}
			else
			{
				s+="0 ";
			}
		}
		s+="\n";
		return s;
    }

    public static class NoSuchElementError extends Error
    {
    	public NoSuchElementError(String msg) {
    		super(msg);
    	}
    } // end of NoSuchElementError

} // end of BitVector




