package cs307.first;
import cs307.threeohseven.*;

/**
 * Descriptor for a BitVector use case. This class provides an
 * implementation of Descriptors suitable for bit vectors.
 *
 * @author -
  **/

public class Descriptor_51 implements Director.Descriptor {

        private final boolean optSpace_;
        private final boolean optSpeed_;
        private final int low_;
        private final int high_;
        private final boolean checked_;


        /**
         * Make a descriptor for bit vectors from <code>low</code> to
         * <code>high</code>.
         * @param low lower bound on active range
         * @param high higher bound on active range
         **/
        public Descriptor_51(int low, int high) {
            optSpace_ = false;
            optSpeed_ = false;
            low_ = low;
            high_ = high;
            checked_ = true;
        }

        /**
         * Make a descriptor for bit vectors from <code>low</code> to
         * <code>high</code>.
         * @paramow lower bound on active range
         * @param high higher bound on active range
         * @param optSpace optimize for space
         * @param optSpeed optimize for speed
         * @param checked  full bound checks required
         **/
        public Descriptor_51(int low,
                          int high,
                          boolean optSpace,
                          boolean optSpeed,
                          boolean checked) {
            optSpace_ = optSpace;
            optSpeed_ = optSpeed;
            low_ = low;
            high_ = high;
            checked_ = checked;
        }

        /**
         * Should storage space be minimized?
         **/
        public boolean optimizeSpace() {
            return optSpace_;
        }
        /**
         * Should speed be maximized?
         **/
        public boolean optimizeSpeed() {
            return optSpeed_;
        }
        /**
         * Lower bound on active range.
         **/
        public int getLowerBound() {
            return low_;
        }
        /**
         * Upper bound on active range.
         **/
        public int getUpperBound() {
            return high_;
        }

        /**
         * Returns true if bounds checks are required for all accesses. If
         * this is set to false an implementation may choose not check some
         * operations. Typically this allows removal of some redundant
         * checks at the risk of overflowing. If the method returns true,
         * the corresponding vectors are strict, otherwise they called
         * relaxed.
		**/
        public boolean boundCheckRequired() {
                return checked_;
        }

        /**
         * Return a short description of the requirements for BitVectors
         * created by this descriptor.
         **/
        public String getDescription() {
            return "BitVector [" +low_+","+high_+"], opt space ="+optSpace_
                +" opt speed ="+optSpeed_;
        }
        /**
         * Return true if the two descriptor are identical.
         **/
        public boolean equals(Object o) {
            if (o == this) return true;
            if (o instanceof Descriptor_51 == false) return false;
            Descriptor_51 descr = (Descriptor_51) o;
            return
                high_ == descr.high_ &&
                low_ == descr.low_ &&
                checked_ == descr.checked_ &&
                optSpeed_ == descr.optSpeed_ &&
                optSpace_ == descr.optSpace_;
        }

    } // end of BitVector.Descriptor
