package cs307.first;

/** 
 * Home-made vector class for use with my bit vector implementation.
 * @author -
 *
 */
public class Vector_51 {
	private int size_;
	private int used_;
	private boolean spaceOpt_;
	private Object[] vector_;
	private int[] vector2_;
	
	/**
	 * default builder takes no arguements
	 * Uses speed-optimized vector
	 */
	public Vector_51()
	{
		vector_ = new Object[10];
		used_=10;
		spaceOpt_=false;
	}
	
	/**
	 * constructor takes argument size, uses speed-optimized vector
	 * @param size integer which sets size of vector
	 */
	public Vector_51(int size)
	{
		size_ = size;
		vector_ = new Object[size];
		used_=10;
		spaceOpt_=false;
	}
	
	/**
	 * constructor takes argument opt, which is a boolean 
	 * @param opt boolean - if true vector is space optimized. If false it is speed optimized.
	 */
	public Vector_51(boolean opt)
	{
		used_=0;
		spaceOpt_ = opt;
		vector_ = new Object[10];
		vector2_ = new int[10];
	}
	
	/**
	 * constructor takes argument opt, which is a boolean, and argument size, which is an int. 
	 * @param opt boolean - if true vector is space optimized. If false it is speed optimized.
	 * @param size int - sets size of vector.
	 */
	public Vector_51(int size, boolean opt)
	{
		this.used_=0;
		this.spaceOpt_ = opt;
		this.vector_ = new Object[size];
		this.vector2_ = new int[size];
	}
	
	/** 
	 * @return size of vector
	 */
	public int size()
	{
		return this.size_;
	}
	
	/**
	 * Searches vector's array for location of the find position.  Useful only for the space optimized vector.  
	 * This is useful because only certain bits are kept track of.
	 * @param find -integer function needs to find.
	 * @return Returns -1 if bit location cannot be found, or the location if it can be.
	 */
	public int searchThrough(int find)
	{
		for(int i=0;i<used_;i++)
		{
			if(vector2_[i]==find)
			{
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * Used to find an object inside the vector's array.
	 * @param i Integer representing the stop in the vector we want to return.
	 * @return Returns the Object at spot i.
	 */
	public Object elementAt(int i)
	{
		int spot=-2;
		if(spaceOpt_==true)
		{
			spot = searchThrough(i);
			if(spot==-1)
				return new Boolean(false);
			else
				return new Boolean(true);
		}
		return vector_[i];
	}
	
	/**
	 * Used to set an object inside the vector's array.
	 * @param i Integer of where we want to place the object.
	 * @param o Object item we want to place in apot i.
	 * @return Returns the Object at spot i.
	 */
	public void setElementAt(Object o,int i)
	{
		int spot=-2;
		if(spaceOpt_==true)
		{
			spot=searchThrough(i);
			//if spot is -1 then element is false and needs to be set
			if(spot==-1)
			{
				add(o,i);
			}
			else if(o.equals(new Boolean(false)))
			{
				remove(i);
			}
			else
			{
				vector_[spot]=o;
			}
		}
		vector_[i]=o;
	}
	
	/**
	 * Add an object to the end of the bit vector array.  
	 * Only useful for space-optimized vector.  
	 * Should NOT be used for speed-optimized vector.
	 * @param o Object to add.
	 * @param i Integer of space to add object to.
	 */
	public void add(Object o, int i)
	{
		if(spaceOpt_==true)
			return;
		if(used_==size_)
		{
			//need to increase
			increase();
		}
		vector_[used_]=o;
		vector2_[used_]=i;
	}
	
	/**
	 * Remove an object from bit vector array.  
	 * Also calls decrease automatically.
	 * Only useful for space-optimized vector.  
	 * Should NOT be used for speed-optimized vector.
	 * @param i Integer of space to remove.
	 */
	public void remove(int i)
	{
		if(spaceOpt_==true)
			return;
		int spot = searchThrough(i);
		//delete spot, move everything over until you get to used_
		for(int j=spot;j<used_+1;j++)
		{
			vector_[j]=vector_[j+1];
			vector2_[j]=vector2_[j+1];
		}
		used_--;
		vector_[used_]=new Boolean(false);
		vector2_[used_]=-2;
		decrease();
	}
	
	/**
	 * Increase the size of the bit vector array by 10.
	 * Only to be used for space-optimized bit vectors.
	 */
	public void increase()
	{
		if(spaceOpt_==true)
			return;
		size_+=10;
		Object[] temp = vector_;
		vector_ = new Object[size_];
		vector_ = temp;
	}
	
	/**
	 * If there are more than 10 free spots in the bit vector array, decrease array by 10.
	 * Only to be used for space-optimized bit vectors.
	 */
	public void decrease()
	{
		if(spaceOpt_==true)
			return;
		if(size_>used_+10)
		{
			size_-=10;
			Object[] temp = vector_;
			vector_ = new Object[size_];
			vector_ = temp;
		}
	}
}
