/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cs307.first;

/**
 * This BitVector class implements the BitVector interface using an array of 
 * booleans
 * @author -
 */
public class BitVectorBig_68 implements BitVector {

    private boolean[] _bitVector;
    private int _low = 0,  _high = -1;
    //private boolean _boundCheck = false;
    
    /**
     * Constructor, creates a bitVector from a low and high range
     * This implementation uses an array of booleans to allow for easy access
     * And to primarily perform SET, and TEST operations
     * @param low Low bound of the bitVector
     * @param high High bound of the bitVector
     */
    public BitVectorBig_68(int low, int high) {
        _low = low;
        _high = high;
        int size = _high - _low + 1;
        if (size < 1) {
            throw new Error("Illegal Bounds");
        }
        _bitVector = new boolean[size];
    }

    /**
     * Constructs a bitVector to be a copy of the given bitvector
     * This implementation uses an array of booleans to allow for easy access
     * And to primarily perform SET, and TEST operations
     * @param bv The bitVector from which the copy is made
     */
    public BitVectorBig_68(BitVector bv) {
        Iterator myIter = bv.iterator();
        _low = myIter.position();
        while (myIter.hasNext()) {
            myIter.skipToSet();
            myIter.getNext();
        }
        _high = myIter.position() - 1;
        int size = _high - _low + 1;
        if (size < 1) {
            throw new Error("Illegal Bounds");
        }
        _bitVector = new boolean[size];
        myIter = bv.iterator();
        myIter.skipToSet();
        while(myIter.hasNext()){
            _bitVector[myIter.position() - _low] = myIter.getNext();
            myIter.skipToSet();
        }
    }

    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return value at <code>index</code> 
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index) {
        if (index >= _low && index <= _high) {
            return _bitVector[index - _low];
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index) {
        if (index >= _low && index <= _high) {
            boolean temp = _bitVector[index - _low];
            _bitVector[index - _low] = true;
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index) {
        if (index >= _low && index <= _high) {
            boolean temp = _bitVector[index - _low];
            _bitVector[index - _low] = false;
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index) {
        if (index >= _low && index <= _high) {
            boolean temp = _bitVector[index - _low];
            _bitVector[index - _low] = !_bitVector[index - _low];
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void and(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = _bitVector[i] && bs.test(i);
        }
    }

    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void or(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = _bitVector[i] || bs.test(i);
        }
    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void xor(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = (_bitVector[i] && !bs.test(i)) || (!_bitVector[i] && bs.test(i));
        }
    }

    /**
    Returns a logically disjoint deep copy of the object.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy() {
        return new BitVectorBig_68(this);
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    public Iterator iterator() {
        return new Iterator_68(this, _low, _high);
    }

    /**
     * Returns the builder that created this <code>BitVector</code>.
     **/
    public Builder builder() {
        return new BuilderBig_68(_low, _high);
    }

    /**
     * Return a representation of this <code>BitVector</code> as a
     * string. The format is implementation specific.
     **/
    public String toString() {
        String out = "Array booleans: \n[";
        for (int i = 0; i < _bitVector.length -1; ++i) {
            out += _bitVector[i] + ",";
            if((i+1)%8 == 0 ){
                out+="\n";
            }
        }
        out+= _bitVector[_high - _low] + "]";
        return out;
    }
}
