/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cs307.first;

/**
 * This BitVector class imlaments the BitVector interface using an array of 
 * ints
 * @author -
 */
public class BitVectorSmall_68 implements BitVector {

    private int[] _bitVector;
    private int _low = 0,  _high = -1;
    //private boolean _boundCheck = false;
    
    /**
     * Constructor, creates a bitVector from a low and high range.
     * This implementation uses an array of ints to pack the data in a small space
     * And to primarily perform AND, OR, and TEST operations
     * @param low Low bound of the bitVector
     * @param high High bound of the bitVector
     */
    public BitVectorSmall_68(int low, int high) {
        _low = low;
        _high = high;
        int size = ((_high - _low) / 31) + 1;
        if (size < 1) {
            throw new Error("Illegal Bounds");
        }
        _bitVector = new int[size];
    }

    /**
     * Constructs a bitVector to be a copy of the given bitvector
     * This implementation uses an array of ints to pack the data in a small space
     * And to primarily perform AND, OR, and TEST operations
     * @param bv The bitVector from which the copy is made
     */
    public BitVectorSmall_68(BitVector bv) {
        Iterator myIter = bv.iterator();
        _low = myIter.position();
        while (myIter.hasNext()) {
            myIter.skipToSet();
            myIter.getNext();
        }
        _high = myIter.position() - 1;
        int size = ((_high - _low) / 31) + 1;
        if (size < 1) {
            throw new Error("Illegal Bounds");
        }
        _bitVector = new int[size];
        BitVectorSmall_68 _bv = (BitVectorSmall_68)bv;
        for(int i=0; i<_bitVector.length; ++i){
            _bitVector[i] = _bv._bitVector[i];
        }
    }

    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return value at <code>index</code> 
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index) {
        if (index >= _low && index <= _high) {
            index -= _low;
            int i = index % 31;
            index = (int) Math.floor(index / (double) 31);
            return find(_bitVector[index], i);
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index) {
        if (index >= _low && index <= _high) {
            index -= _low;
            int i = index % 31;
            index = (int) Math.floor(index / (double) 31);
            boolean temp = find(_bitVector[index], i);
            if (!temp) {
                _bitVector[index] = insert(_bitVector[index], i);
            }
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index) {
        if (index >= _low && index <= _high) {
            index -= _low;
            int i = index % 31;
            index = (int) Math.floor(index / (double) 31);
            boolean temp = find(_bitVector[index], i);
            if (temp) {
                _bitVector[index] = remove(_bitVector[index], i);
            }
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index) {
        if (index >= _low && index <= _high) {
            index -= _low;
            int i = index % 31;
            index = (int) Math.floor(index / (double) 31);
            boolean temp = find(_bitVector[index], i);
            if (!temp) {
                _bitVector[index] = insert(_bitVector[index], i);
            } else {
                _bitVector[index] = remove(_bitVector[index], i);
            }
            return temp;
        }
        throw new BitVector.NoSuchElementError("Index out of range");
    }

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void and(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        BitVectorSmall_68 _bs = (BitVectorSmall_68)bs;
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = _bitVector[i] & _bs._bitVector[i];
        }
    }

    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void or(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        BitVectorSmall_68 _bs = (BitVectorSmall_68)bs;
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = _bitVector[i] | _bs._bitVector[i];
        }
    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void xor(BitVector bs) {
        if (!bs.builder().compatible(this.builder())) {
            throw new ImplementationError("Incompatible objects");
        }
        BitVectorSmall_68 _bs = (BitVectorSmall_68)bs;
        for (int i = 0; i < _bitVector.length; ++i) {
            _bitVector[i] = _bitVector[i] ^ _bs._bitVector[i];
        }
    }

    /**
    Returns a logically disjoint deep copy of the object.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy() {
        return new BitVectorSmall_68(this);
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    public Iterator iterator() {
        return new Iterator_68(this, _low, _high);
    }

    /**
     * Returns the builder that created this <code>BitVector</code>.
     **/
    public Builder builder() {
        return new BuilderSmall_68(_low, _high);
    }

    /**
     * Return a representation of this <code>BitVector</code> as a
     * string. The format is implementation specific.
     **/
    public String toString() {
        String out = "Array ints: \n[";
        Iterator iter = this.iterator();
        out += iter.getNext();
        int i=1;
        while(iter.hasNext()){
            out += ", ";
            if(i%8 == 0){
                out += "\n";
            }
            ++i;
            out += iter.getNext();
        }
        out += "]";
        return out;
    }
    // </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc="Private Methods">
    /**
     * Given an integer it is parsed to find the boolean value compressed within
     * the 32 bits representing the integer at the given index
     * @param value Integer representing a compressed bitVector
     * @param index The index of the bit in question
     * @return The boolean value contained at the index in question
     */
    private boolean find(int value, int index) {
        for (int i = 30; i >= 0; --i) {
            if (value == 0) {
                return false;
            } else if (value >= Math.pow(2, i)) {
                if (index == i) {
                    return true;
                }
                value -= Math.pow(2, i);
            } else if (i == index) {
                return false;
            }
        }
        // Unknown error.
        return false;
    }

    private int insert(int value, int index) {
        return value + (int) Math.pow(2, index);
    }

    private int remove(int value, int index) {
        return value - (int) Math.pow(2, index);
    }
}
