package test.first.u68;


import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import cs307.first.BitVector;
import cs307.first.Director_68;
import cs307.first.Director.Builder;

public class OptimizationTesting {

	enum Ops {OR, AND, XOR, TEST, SET, UNSET, NEGATE};
	
	private final int UL = 127, LL = 0;
	private int _uLimit, _lLimit;	
	private int _case1Size = 2000, _case2Size = 100;
	private BitVector[] testVectors;
	private Director_68 testDirector;
	
	@Before
	public void setUp() throws Exception {
		testDirector = new Director_68();
		//testDirector._lowerLimit = LL;
		//testDirector._upperLimit = UL;
	}

	@After
	public void tearDown() throws Exception {
		testDirector = null;
	}
	
	@Test
	public void testUseCase1_BVSpace()	{ // SMALL, AND, OR, TEST
		System.out.println("Use Case 1 - Space Imp.\n-----------------------");
		_uLimit = 100;
		_lLimit = 0;
		makeVectors(_lLimit, _uLimit, true, false, _case1Size);		
		
		doAlternate();
		
		// Time OR
		testSet(Ops.OR);
		
		doAlternate(1);
		
		// Time AND
		testSet(Ops.AND);
		
		// Time TEST
		testSet(Ops.TEST);
		System.out.println();
	}
	
	@Test
	public void testUseCase1_BVSpeed()	{ // SMALL, AND, OR, TEST
		System.out.println("Use Case 1 - Speed Imp.\n-----------------------");
		_uLimit = 100;
		_lLimit = 0;
		makeVectors(_lLimit, _uLimit, false, true, _case1Size);
		
		doAlternate();
		
		// Time OR
		testSet(Ops.OR);
		
		doAlternate(1);
		
		// Time AND
		testSet(Ops.AND);
		
		// Time TEST
		testSet(Ops.TEST);
	}
	
	@Test
	public void testUseCase2_BVSpace() { // LARGE, SET, TEST
		System.out.println("Use Case 2 - Space Imp.\n-----------------------");
		_uLimit = 6000;
		_lLimit = 0;
		//testDirector._upperLimit = _uLimit + 100;
		makeVectors(_lLimit, _uLimit, true, false, _case2Size);		
		// Time SET
		testSet(Ops.SET);

		// Time TEST
		testSet(Ops.TEST);
		System.out.println();
	}
	
	@Test
	public void testUseCase2_BVSpeed() { // LARGE, SET, TEST
		System.out.println("Use Case 2 - Space Imp.\n-----------------------");
		_uLimit = 600000;
		_lLimit = 0;
		//testDirector._upperLimit = _uLimit + 100;
		makeVectors(_lLimit, _uLimit, false, true, _case2Size);		
		// Time SET
		testSet(Ops.SET);

		// Time TEST
		testSet(Ops.TEST);
		System.out.println();
	}
	
	private void testSet(Ops func){
		long _start, _elapsed ;
		boolean print = true;
		_start = System.currentTimeMillis();
		switch(func){
		case AND:
			doAND();
			System.out.print("Test AND: ");
			break;
		case OR:
			doOR();
			System.out.print("Test OR: ");
			break;
		case TEST:
			doTEST();
			System.out.print("Test TEST: ");
			break;
		case SET:
			doSET();
			System.out.print("Test SET: ");
			break;
		default:
			print = false;
			break;
		}
		_elapsed = System.currentTimeMillis() - _start;
		if(print){
			System.out.println(_elapsed);
		}
	}
	
	private void testSet(Ops[] funcs){
		long _start, _elapsed ;
		boolean print = true;
		for(int i=0; i<funcs.length; ++i){
			print = true;
			_start = System.currentTimeMillis();
			switch(funcs[i]){
			case AND:
				doAND();
				System.out.print("Test AND: ");
				break;
			case OR:
				doOR();
				System.out.print("Test OR: ");
				break;
			case TEST:
				doTEST();
				System.out.print("Test TEST: ");
				break;
			case SET:
				doSET();
				System.out.print("Test SET: ");
				break;
			default:
				print = false;
				break;
			}
			_elapsed = System.currentTimeMillis() - _start;
			if(print){
				System.out.println(_elapsed);
			}
		}
	}
	
	private void doOR(){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=0; j<testVectors.length; ++j){
				testVectors[i].or(testVectors[j]);
			}
		}
	}	
	
	
	private void doAND(){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=0; j<testVectors.length; ++j){
				testVectors[i].and(testVectors[j]);
			}
		}
	}
	
	private void doTEST(){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=_lLimit; j<=_uLimit; ++j){
				testVectors[i].test(j);
			}
		}
	}
	
	private void doSET(){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=_lLimit; j<=_uLimit; ++j){
				testVectors[i].set(j);
			}
		}
	}
	
	private void doAlternate(){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=_lLimit; j<=_uLimit; ++j){
				if(j%2 == 0){
					testVectors[i].set(j);
				}else{
					testVectors[i].unset(j);
				}
			}
		}
	}
	
	private void doAlternate(int offset){
		for(int i=0; i<testVectors.length; ++i){
			for(int j=_lLimit; j<=_uLimit; ++j){
				if((j+offset)%2 == 0){
					testVectors[i].set(j);
				}else{
					testVectors[i].unset(j);
				}
			}
		}
	}
	
	private void makeVectors(int ul, int ll, boolean space, boolean speed, int num){
		BitVector.Descriptor temp1 = new BitVector.Descriptor(ul, ll, space, speed, true);
		Builder temp2 = testDirector.makeBuilder(temp1);
		testVectors = new BitVector[num];
		for(int i=0; i<testVectors.length; ++i){
			testVectors[i] = (BitVector)temp2.build();
		}
	}	
}
