package test.first.u68;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import cs307.first.BitVector;
import cs307.first.BitVectorSmall_68;
import cs307.first.BitVectorBig_68;
import cs307.first.Director;
import cs307.first.Director_68;
import cs307.first.BitVector.NoSuchElementError;
import cs307.first.Director.Builder;

public class TestBigBV {

	private int _limit = 35536; // Power of 2 should be problematic
							 // NOTE: do not reduce below 40 otherwise test will enter an ifninite loop
	
	private Director testDirector = new Director_68();
	private BitVector.Descriptor testDescriptor = new BitVector.Descriptor(0, _limit, false, true, true);
	private Builder testBuilder = testDirector.makeBuilder(testDescriptor);
	private BitVector[] testVectors = new BitVector[4];
	
	@Before
	public void setUp() throws Exception {
		for(int i=0; i<testVectors.length; ++i){
			testVectors[i] = (BitVector)testBuilder.build();
		}
	}

	@After
	public void tearDown() throws Exception {
		for(int i=0; i<testVectors.length; ++i){
			testVectors[i] = null;
		}
	}

	@Test
	public void testBitVectorSpeed_kdosenbaIntInt() {
		testVectors[0] = new BitVectorSmall_68(0, _limit);
		assertNotNull(testVectors[0]);
	}

	@Test
	public void testBitVectorSpeed_kdosenbaBitVector() {
		testVectors[0].set(0);
		testVectors[0].set(1);
		testVectors[0].set(_limit-1);
		testVectors[0].set(_limit);
		for(int i=2; i<=_limit-2; ++i){
			if(Math.random() > .5){
				testVectors[0].set(i);
			}			
		}		
		testVectors[1] = new BitVectorBig_68(testVectors[0]);
		assertNotNull(testVectors[0]);
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i) == testVectors[1].test(i));
		}
	}

	@Test
	public void testTest() {
		try { // out of bounds
			testVectors[0].test(-1);
			fail("-1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			assertFalse(testVectors[0].test(0));			
        } 
        catch (NoSuchElementError success) {
        	fail("0, Should not have rased an exception");
        } 
       
        try { // out of bounds
			testVectors[0].test(_limit+1);
			fail("_limit+1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			assertFalse(testVectors[0].test(_limit));			
        } 
        catch (NoSuchElementError success) {
        	fail("_limit, Should not have rased an exception");
        } 
        
        try { // middle
			assertFalse(testVectors[0].test(_limit/2));			
        } 
        catch (NoSuchElementError success) {
        	fail("_limit/2, Should not have rased an exception");
        } 
        
        try { // out of range
			assertFalse(testVectors[0].test(_limit* 2));		
			fail("_limit * 2, Should not have rased an exception");
        } 
        catch (NoSuchElementError success) {        	
        } 
        
        int[] myList = new int[30];
        for(int i=0; i<myList.length; ++i){
        	myList[i] = (int)(Math.random()*_limit);
        }
        for(int i=0; i<myList.length; ++i){
        	testVectors[0].set(myList[i]);
        }
        boolean temp = false;
        for(int i=0; i<=_limit; ++i){
        	temp = false;
        	try { // in range    				
    			for(int j=0; j<myList.length; ++j){
    				if(i == myList[j]){
    					assertTrue(i+" should have been true", testVectors[0].test(i));
    					temp = true;
    					break;
    				}
    			}
    			if(!temp){
    				assertFalse(testVectors[0].test(i));
    			}
            } 
            catch (NoSuchElementError success) {
            	fail("Non of these should produce an exception");
            } 
        }
	}

	@Test
	public void testSet() {	
		
		// Bound Checks
		try { // out of bounds
			testVectors[1].set(-1);
			fail("-1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].set(0);			
        } 
        catch (NoSuchElementError success) {
        	fail("0, Should not have rased an exception");
        } 
       
        try { // out of bounds
			testVectors[1].set(_limit+1);
			fail("_limit+1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].set(_limit);			
        } 
        catch (NoSuchElementError success) {
        	fail("_limit, Should not have rased an exception");
        } 
        
        // Function test
        for(int i=0; i<=_limit; ++i){
			try{
				testVectors[0].set(i);
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
		for(int i=0; i<=_limit; ++i){
			try{
				assertTrue(testVectors[0].test(i));
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
	}

	@Test
	public void testUnset() {	
		
		// Bound Checks
		try { // out of bounds
			testVectors[1].unset(-1);
			fail("-1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].unset(0);			
        } 
        catch (NoSuchElementError success) {
        	fail("0, Should not have rased an exception");
        } 
       
        try { // out of bounds
			testVectors[1].unset(_limit+1);
			fail("_limit+1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].unset(_limit);			
        } 
        catch (NoSuchElementError success) {
        	fail("_limit, Should not have rased an exception");
        } 
        
        // Function test
        for(int i=0; i<=_limit; ++i){
			testVectors[0].set(i);			
		}
		for(int i=0; i<=_limit; ++i){
			try{
				testVectors[0].unset(i);
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
		for(int i=0; i<=_limit; ++i){
			try{
				assertFalse(testVectors[0].test(i));
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
	}

	@Test
	public void testNegate() {		
		
		// Bound Checks
		try { // out of bounds
			testVectors[1].negate(-1);
			fail("-1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].negate(0);			
        } 
        catch (NoSuchElementError success) {
        	fail("0, Should not have rased an exception");
        } 
       
        try { // out of bounds
			testVectors[1].negate(_limit+1);
			fail("_limit+1, Should have rased an exception");
        } 
        catch (NoSuchElementError success) {
        } 
        
        try { // at edge
			testVectors[1].negate(_limit);			
        } 
        catch (NoSuchElementError success) {
        	fail("_limit, Should not have rased an exception");
        } 
        
        // Function Test
        for(int i=0; i<=_limit; ++i){
			try{
				testVectors[0].negate(i);
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i));			
		}
		
		for(int i=0; i<=_limit; ++i){
			try{
				testVectors[0].negate(i);
			}
			catch(NoSuchElementError success) {
            	fail(i+" should produce an exception");
            } 
		}
		for(int i=0; i<=_limit; ++i){
			assertFalse(testVectors[0].test(i));			
		}
	}

	@Test
	public void testAnd() {
		for(int i=0; i<=_limit; ++i){
			testVectors[0].set(i);
			testVectors[1].set(i);
		}
		testVectors[0].and(testVectors[1]); // 1 and 1
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i));
		}
		testVectors[0].and(testVectors[2]); // 1 and 0
		for(int i=0; i<=_limit; ++i){
			assertFalse(testVectors[0].test(i));
		}
		testVectors[0].and(testVectors[1]); // 0 and 1
		for(int i=0; i<=_limit; ++i){
			assertFalse(testVectors[0].test(i));
		}
		testVectors[0].and(testVectors[2]); // 0 and 0
		for(int i=0; i<=_limit; ++i){
			assertFalse(testVectors[0].test(i));
		}
	}

	@Test
	public void testOr() {
		for(int i=0; i<=_limit; ++i){
			testVectors[0].set(i);
			testVectors[1].set(i);
		}
		testVectors[0].or(testVectors[1]); // 1 or 1
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i));
		}
		testVectors[0].or(testVectors[2]); // 1 or 0
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i));
		}
		testVectors[2].or(testVectors[3]); // 0 or 0
		for(int i=0; i<=_limit; ++i){
			assertFalse(testVectors[2].test(i));
		}
		testVectors[2].or(testVectors[1]); // 0 or 1
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[2].test(i));
		}
	}

	@Test
	public void testXor() {
		for(int i=0; i<=_limit; ++i){
			testVectors[0].set(i);
			testVectors[1].set(i);
		}
		testVectors[0].xor(testVectors[1]); // 1 xor 1
		for(int i=0; i<=_limit; ++i){
			assertFalse(i+" should not have failed",testVectors[0].test(i));
		}
		testVectors[0].xor(testVectors[2]); // 0 xor 0
		for(int i=0; i<=_limit; ++i){
			assertFalse(i+" should not have failed",testVectors[0].test(i));
		}
		testVectors[2].xor(testVectors[1]); // 0 xor 1
		for(int i=0; i<=_limit; ++i){
			assertTrue(i+" should not have failed",testVectors[2].test(i));
		}
		testVectors[1].xor(testVectors[3]); // 1 xor 0
		for(int i=0; i<=_limit; ++i){
			assertTrue(i+" should not have failed",testVectors[1].test(i));
		}
	}

	@Test
	public void testCopy() {
		testVectors[0].set(0);
		testVectors[0].set(1);
		testVectors[0].set(_limit-1);
		testVectors[0].set(_limit);
		for(int i=2; i<=_limit-2; ++i){
			if(Math.random() > .5){
				testVectors[0].set(i);
			}			
		}		
		testVectors[1] = testVectors[0].copy();
		assertNotNull(testVectors[0]);
		for(int i=0; i<=_limit; ++i){
			assertTrue(testVectors[0].test(i) == testVectors[1].test(i));
		}
	}

	@Test
	public void testIterator() {
		int size = 40;
		int[] myList = new int[size];
		// Bound tests
		myList[0] = 0;
		myList[1] = 1;
		myList[2] = _limit-1;
		myList[3] = _limit;
		myList[4] = 29;
		myList[5] = 30;
		myList[6] = 31;
		myList[7] = 32;
		myList[8] = 33;
		myList[9] = 34;
		boolean t = false;
        for(int i=10; i<myList.length; ++i){
        	do{
        		t = false;
        		myList[i] = (int)(Math.random()*_limit);
        		for(int j=0; j<i; ++j){
        			if(myList[i] == myList[j]){
        				t = true;
        				break;
        			}
        		}
        	}while(t);
        }
        for(int i=0; i<=_limit; ++i){
        	testVectors[1].set(i);
        	for(int j=0; j<myList.length; ++j){
        		if(i==myList[j]){
        			System.out.print(i+", ");
        			testVectors[0].set(i);
        			testVectors[1].unset(i);
        		}
        	}
        }
        System.out.println();
        BitVector.Iterator iter = testVectors[0].iterator();
        assertNotNull(iter); // Test its creation
        
        // Test Has Next
        assertTrue(iter.hasNext()); 
        for(int i=0; i<=_limit; ++i){
        	iter.getNext();        	
        }
        assertFalse(iter.hasNext()); 
        
        iter = testVectors[0].iterator();
        
        // Test getNext
        boolean temp = false;
        int i = 0;
        while(iter.hasNext()){
        	temp = false;
        	for(int j=0; j<myList.length; ++j){
				if(i == myList[j]){
					assertTrue(i+" should have been true", iter.getNext());
					temp = true;
					break;
				}
			}
        	if(!temp){
				assertFalse(iter.getNext());
			}
        	++i;
        }
        
        iter = testVectors[0].iterator();
        boolean[] validate = new boolean[size];
        for(int k=0; k<size; ++k){
        	validate[k] = false;
        }
        
        // Test skiptoset
        while(iter.hasNext()){
        	temp = false;
        	i = iter.skipToSet();
        	System.out.print("i: "+ i);
        	for(int j=0; j<myList.length; ++j){
				if(i == myList[j]){
					System.out.println("    j: "+ j);
					assertTrue(i+" should have been true", iter.getNext());
					validate[j] = true;
					temp = true;
					break;
				}
			}
        	if(!temp){
				fail("Found unknown set variable at "+ i);
			}
        }
        
        for(int k=0; k<size; ++k){
        	assertTrue("Did not find set bit "+myList[k], validate[k]);
        }
        
        
        iter = testVectors[1].iterator();
        for(int k=0; k<size; ++k){
        	validate[k] = false;
        }
        // Test skiptounset
        while(iter.hasNext()){
        	temp = false;
        	i = iter.skipToUnset();
        	for(int j=0; j<myList.length; ++j){
				if(i == myList[j]){
					assertFalse(i+" should have been false", iter.getNext());
					validate[j] = true;
					temp = true;
					break;
				}
			}
        	if(!temp){
				fail("Found unknown unset variable at "+ i);
			}
        }
        
        for(int k=0; k<size; ++k){
        	assertTrue("Did not find unset bit "+myList[k], validate[k]);
        }
        
        // Test position
        iter = testVectors[0].iterator();
        
        // Test getNext
        while(iter.hasNext()){
        	temp = false;
        	iter.skipToSet();
        	i = iter.position();
        	for(int j=0; j<myList.length; ++j){
				if(i == myList[j]){
					temp = true;
					break;
				}
			}
        	if(!temp){
				fail("Unexpected Position " + i);
			}
        	iter.getNext();
        }
	}

	@Test
	public void testBuilder() {
		for(int i=0; i<testVectors.length; ++i){
			assertEquals("Builders should have been equal", testBuilder, testVectors[i].builder());
		}
	}

	@Test
	public void testToString() {
		if(_limit<1024){
			assertTrue(testVectors[0].toString().startsWith("Array booleans:"));
		}else
		{
			fail("Not Implemented for large imput size");
		}
	}

}
