package cs307.first;

import threeohseven.BitVector;
import threeohseven.ImplementationError;

/**
 * A general implementation of a BitVector.  It uses an array of boolean values.
 * This gives simple code that has fast read and write times, but uses more memory than a
 * packed version.
 */
public class BitVectorGeneral_7 implements BitVector {
	private BitVector.Builder b;
	private int low, high;
	private boolean[] data;
	private boolean check;
	
	public BitVectorGeneral_7 (BitVector.Builder b, int low, int high, boolean check) {
		this.b = b;
		data = new boolean [high-low+1];
		this.low = low;
		this.high = high;
		this.check = check;
	}
	
	public boolean test(int index) {
		if (check) {
			if (index > high || index < low)
				throw new ImplementationError ("Out of bounds");
		}
		
		return data[index-low];
	}
	
	public boolean set(int index) {
		if (check) {
			if (index > high || index < low)
				throw new ImplementationError ("Out of bounds");
		}
		
		boolean old_val = data[index-low];

		data[index-low] = true;

		return old_val;
	}
	
	public boolean unset(int index) {
		if (check) {
			if (index > high || index < low)
				throw new ImplementationError ("Out of bounds");
		}

		boolean old_val = data[index-low];

		data[index-low] = false;

		return old_val;
	}
	
	public boolean negate(int index) {
		if (check) {
			if (index > high || index < low)
				throw new ImplementationError ("Out of bounds");
		}

		boolean old_val = data[index-low];
		
		if (data[index-low])
			data[index-low] = false;
		else
			data[index-low] = true;
		
		return old_val;
	}
	
	public void and(BitVector bs) {
		if (check) {
			if (!b.compatible (bs.builder ()))
				throw new ImplementationError ("Non-compatible objects");
		}
		
		for (int i = low; i <= high; i++)
			data[i-low] = data[i-low] && bs.test(i);
	}
	
	public void or(BitVector bs) {
		if (check) {
			if (!b.compatible (bs.builder ()))
				throw new ImplementationError ("Non-compatible objects");
		}
		
		for (int i = low; i <= high; i++)
			data[i-low] = data[i-low] || bs.test(i);
	}
	
	public void xor(BitVector bs) {
		if (check) {
			if (!b.compatible (bs.builder ()))
				throw new ImplementationError ("Non-compatible objects");
		}
		
		for (int i = low; i <= high; i++)
			data[i-low] = data[i-low] ^ bs.test(i);
	}
	
	public BitVector copy() {
		BitVectorGeneral_7 new_bv = (BitVectorGeneral_7) b.buildBitVector ();
		
		for (int i = low; i <= high; i++)
			new_bv.data[i-low] = data[i-low];
			
		return (BitVector) new_bv;
	}
	
	public BitVector.Iterator iterator() {
		return new Iterator (this);
	}
	
	public Builder builder() {
		return b;
	}
	
	public String toString() {
		String ret = "";
		for (int i = high; i >= low; i--) {
			if (data[i-low])
				ret = ret + "1";
			else
				ret = ret + "0";
		}
		return ret;
	}
	
	public static class Iterator implements BitVector.Iterator {
		private int pos;
		private BitVectorGeneral_7 bv;
		
		public Iterator (BitVectorGeneral_7 bv) {
			this.bv = bv;
			pos = bv.low;
		}
		
		public boolean getNext () {
			return bv.data[pos++];
		}
		
		public boolean hasNext () {
			return (pos <= bv.high);
		}
		
		public int position () {
			return pos;
		}
		
		public int skipToSet () {
			while (pos <= bv.high) {
				if (bv.data[pos]) {
					return pos;
				}
				pos++;
			}
			return pos;
		}
		
		public int skipToUnset () {
			while (pos <= bv.high) {
				if (!bv.data[pos]) {
					return pos;
				}
				pos++;
			}
			return pos;
		}
	}
}

