package test.first.u72;

import junit.framework.*;
import junit.textui.*;
import threeohseven.*;
import threeohseven.BitVector.*;
import java.util.Random;

public class TestBitvector extends TestCase
{
  protected Director director;
  protected BitVector.Descriptor small;
  protected BitVector.Builder builderSmall;
  protected BitVector b;

  protected BitVector.Descriptor big;
  protected BitVector.Builder builderBig;
  protected BitVector bs;

  /***************************************************
   *  setUp runs at the start and end of every test
   *  and ensures that there is a fresh copy of 
   *  everything for each iteration.
   ***************************************************/

  protected void setUp()
  {
    director = new Director_u72();
    small = new BitVector.Descriptor(0, 100, false, true, true);
    big = new BitVector.Descriptor(0, 10000000, false, true, true);
    builderSmall = (BitVector.Builder) director.makeBuilder(small);
    builderBig = (BitVector.Builder) director.makeBuilder(big);
    b = (BitVector)builderSmall.build();
    bs = (BitVector)builderBig.build();
  }

  /***************************************************
   *  tearDown is called at the end of each test to 
   *  ensure that all "global" variables are clean
   ***************************************************/

  protected void tearDown()
  {
    b = null;
    bs = null;
  }

  /***************************************************
   *   testSet3pass is a simple test to ensure that 
   *   the function set is working properly
   ***************************************************/

  public void testSet3pass()
  {
    b.set(3);
    Assert.assertEquals("Test Set true", b.test(3), true);
  }

  /***************************************************
   *   testSet3fail is set up to return a false with
   *   the method test.
   ***************************************************/

  public void testSet3fail()
  {
    b.set(3);
    Assert.assertEquals("Test Set false", b.test(2), false);
  }

  /***************************************************
   *   testBigXor tests the large set of BitVectors 
   *   and ensures that if a BitVector is xor-ed with
   *   itself will return all false.
   ***************************************************/

  public void testBigXor()
  {
    BitVector bq;
    Random rand = new Random();
    Iterator it = bs.iterator();
    int j = ((BitVector_u72)bs).getLow();
    for( int i = 0; i < 100; i++ )
    {
      bs.set(rand.nextInt(10000000));
    }
    bq = bs.copy();
    bs.xor(bq);
    while(it.hasNext())
    {
      Assert.assertEquals("Test xor", bs.test(j), false);
      j++;
      it.getNext();
    }
    
  }

  /***************************************************
   *   testSmallXor is the exact same as testBigXor
   *   except that it is run with a small BitVector
   ***************************************************/

  public void testSmallXor()
  {
    BitVector bq;
    Random rand = new Random();
    Iterator it = b.iterator();
    int j = ((BitVector_u72)b).getLow();
    for( int i = 0; i < 75; i++ )
    {
      b.set(rand.nextInt(100));
    }
    bq = b.copy();
    b.xor(bq);
    while(it.hasNext())
    {
      Assert.assertEquals("Test xor", b.test(j), false);
      j++;
      it.getNext();
    }
    
  }

  /***************************************************
   *   testAndOrBig tests the methods "and" and "or"
   *   Two BitVectors are made in which they are 
   *   negations of each other.  the method and should
   *   return all false and the method or should
   *   return all true.
   ***************************************************/

  public void testAndOrBig()
  {
    BitVector bq;
    bq = bs.copy();

    for( int i = 0; i <= 10000000; i++ )
    {
      if( i % 2 == 0)
        bs.set(i);
      else
        bq.set(i);
    }
    BitVector bTemp;
    bTemp = bs.copy();
    bTemp.and(bq);

    Iterator it = bs.iterator();
    int j = 0;
    while(it.hasNext())
    {
      Assert.assertEquals("Test and", bTemp.test(j), false);
      j++;
      it.getNext();
    }
    bTemp = bq.copy();
    bTemp.or(bs);
    it = bq.iterator();
    j = 0;
    while(it.hasNext())
    {
      Assert.assertEquals("Test and", bTemp.test(j), true);
      j++;
      it.getNext();
    }
    
  }

  /***************************************************
   *   testAndOrSmall is the same as testAndOrBig with
   *   the exception that the small BitVector is used
   *   as opposed to the larger one.
   ***************************************************/

  public void testAndOrSmall()
  {
    BitVector bq;
    bq = b.copy();

    for( int i = 0; i <= 100; i++ )
    {
      if( i % 2 == 0)
        b.set(i);
      else
        bq.set(i);
    }
    BitVector bTemp;
    bTemp = b.copy();
    bTemp.and(bq);

    Iterator it = b.iterator();
    int j = 0;
    while(it.hasNext())
    {
      Assert.assertEquals("Test and", bTemp.test(j), false);
      j++;
      it.getNext();
    }
    bTemp = bq.copy();
    bTemp.or(b);
    it = bq.iterator();
    j = 0;
    while(it.hasNext())
    {
      Assert.assertEquals("Test and", bTemp.test(j), true);
      j++;
      it.getNext();
    }
    
  }

  public static Test suite()
  {
    return new TestSuite(TestBitvector.class);
  }

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }
  
}
