import org.junit.After;
import org.junit.Before;

import threeohseven.BitVector;
import threeohseven.Director;
import cs307.first.BitVector_77;
import cs307.first.Director_77;
import junit.framework.*;

public class testIterator extends TestCase {
	
	Director director;
	BitVector.Descriptor small;
	// Create a builder
	BitVector.Builder builder;		
	BitVector_77 bs;
	BitVector.Iterator iter;
	
	public testIterator(String name){
		super(name);
	}
	@Before
	public void setUp() throws Exception {
		director = new Director_77();
		small = new BitVector.Descriptor(0, 31, false, true, true);
		// Create a builder
		builder	= (BitVector.Builder) director.makeBuilder(small);
		
		bs =  (BitVector_77)builder.build();
		iter = bs.iterator();
	}

	@After
	public void tearDown() throws Exception {
		director = null;
		small = null;
		builder = null;
		bs = null;
		iter = null;
		
		
	}
	
	//the following tests test the iterator class
	public void testSimpleTest(){
		
		
		while(iter.hasNext()){
			System.out.println(iter.getNext());
		
		}
		assertEquals(iter.position(),32);//makes sure that the .hasNext() stops when it should.
	}
	
	public void testSkipToSet(){
		BitVector.Iterator iter2;
		bs.set(15);
		iter2 = bs.iterator();
		iter2.skipToSet();

		assertEquals(iter2.position(),15);
	}
	
	public void testSkipToUnset(){
		BitVector.Iterator iter2;
		bs.set(0);
		bs.set(1);
		bs.set(2);
		bs.set(4);
		iter2 = bs.iterator();
		iter2.skipToUnset();
		
		
		assertEquals(iter2.position(),3);
	}
	
	public void testSkipToSetAllFalse(){
		
		BitVector.Iterator iter2;
		//bs.set(15);
		iter2 = bs.iterator();
		iter2.skipToSet();

		assertEquals(iter2.position(),32);
	}
	
	public void testSkipToUnsetAllTrue(){
		BitVector.Iterator iter2;
		//set all bits to true;
		bs.set(0);
		bs.set(1);
		bs.set(2);
		bs.set(3);
		bs.set(4);
		bs.set(5);
		bs.set(6);
		bs.set(7);
		bs.set(8);
		bs.set(9);
		bs.set(10);
		bs.set(11);
		bs.set(12);
		bs.set(13);
		bs.set(14);
		bs.set(15);
		bs.set(16);
		bs.set(17);
		bs.set(18);
		bs.set(19);
		bs.set(20);
		bs.set(21);
		bs.set(22);
		bs.set(23);
		bs.set(24);
		bs.set(25);
		bs.set(26);
		bs.set(27);
		bs.set(28);
		bs.set(29);
		bs.set(30);
		bs.set(31);
		
		iter2 = bs.iterator();
		iter2.skipToUnset();
		
		
		assertEquals(iter2.position(),32);
	}
	
	

	
	//the following tests test the bitvector class.
	
	public void testSet(){
		bs.set(0);
		assertEquals(bs.test(0),true);
	}
	
	public void testUnset(){
		bs.set(0);
		bs.unset(0);
		assertEquals(bs.test(0),false);
	}
	
	public void testNegate(){
		assertEquals(bs.test(0),false);
		bs.negate(0);
		assertEquals(bs.test(0),true);
		bs.negate(0);
		assertEquals(bs.test(0),false);


	}
	
	public void testCopy(){
		BitVector_77 bt = null;
		bs.set(3);
		bs.set(5);
		bt = (BitVector_77) bs.copy();
		assertEquals(bt.test(0),false);
		assertEquals(bt.test(1),false);
		assertEquals(bt.test(3),true);
		assertEquals(bt.test(5),true);
		assertEquals(bt.low_,bs.low_);
		assertEquals(bt.high_,bs.high_);
		assertEquals(bt.strict,bs.strict);
		assertEquals(bt.optSpace_,bs.optSpace_);
		assertEquals(bt.optSpeed_,bs.optSpeed_);
		
	}
	
	public void testAnd(){
		BitVector_77 bt = null;
		bt = new BitVector_77(0, 31, false, false, false);
		
		bt.set(0);//both
		bs.set(0);
		bt.set(1);//only bt
		bs.set(2);//only bs
		
		bt.and(bs);
		
		assertEquals(bt.test(0),true);
		assertEquals(bt.test(1),false);
		assertEquals(bt.test(2),false);
		assertEquals(bt.test(3),false);
	}
	
	public void testOr(){
		BitVector_77 bt = null;
		bt = new BitVector_77(0, 31, false, false, false);
		
		bt.set(0);//both
		bs.set(0);
		bt.set(1);//only bt
		bs.set(2);//only bs
		
		bt.or(bs);
		
		assertEquals(bt.test(0),true);
		assertEquals(bt.test(1),true);
		assertEquals(bt.test(2),true);
		assertEquals(bt.test(3),false);
	}
	
	public void testXor(){
		BitVector_77 bt = null;
		bt = new BitVector_77(0, 31, false, false, false);
		
		bt.set(0);//both
		bs.set(0);
		bt.set(1);//only bt
		bs.set(2);//only bs
		
		bt.xor(bs);
		
		assertEquals(bt.test(0),false);
		assertEquals(bt.test(1),true);
		assertEquals(bt.test(2),true);
		assertEquals(bt.test(3),false);
	}
	
	//the next test is for the builder class
	public void testBuilderTrue(){
		final BitVector.Builder builder2
		= (BitVector.Builder) director.makeBuilder(small);
		
		assertEquals(builder2.equals(builder),true);
		assertEquals(builder2.compatible(builder),true);
	}
	
	public void testBuilderFalse(){
		final BitVector.Descriptor smaller
		= new BitVector.Descriptor(0, 15, false, true, true);
		final BitVector.Builder builder2
		= (BitVector.Builder) director.makeBuilder(smaller);
		
		assertEquals(builder2.equals(builder),false);
		assertEquals(builder2.compatible(builder),false);
	}
}
