/**
 * The Director interface for 307-PA1
 **/
package threeohseven;

/**
 * A Director is responsible for creating builders. This is an
 * implementation of the Builder pattern from the GoF which exposes the
 * builder objects. This version of the pattern is a general interface for
 * directors which has not been specialized to BitVectors. The advantage of
 * remaining non-specific is that we can use this interface in different
 * context, the drawback is that additional typecasts will be
 * required. Since builders are rarely constructed in a tight loop the
 * performance overhead of the casts is deemed acceptable.
 *
 * @author -
 * @version 1.0.0
 **/
public interface Director {

    /**
     * Create a builder from a descriptor.
     * @param dd non-null descriptor 
     * @throws ImplementationError if there
     * is no suitable implementation for this descriptor
     **/
    public Builder makeBuilder(Descriptor dd);

    /**
     * The Builder interface returns constructed objects. This is the
     * generic version which return values of Object type. Implementations
     * may choose to provide more specific methods.
     *
     * @author -
     **/
    public interface Builder {
	/**
	 * Returns a freshly created object.
	 * @return a non-null object
	 **/
	public Object build();
    }

    /**
     * Describes requirements for the objects to be built. The generic
     * interface only has an information method in it. Specific
     * implementations will provide richer interfaces.
     * @author -
     **/
    public interface Descriptor {
	/**
	 * Returns a string describing the nature of this descriptor.
	 **/
	public String getDescription();

    }

}// End of Director
