/**
 * The BitVectorBig_u83 class for 307-PA1, optimized for large-set
 * The bits are members of a boolean array for quick and easy get/set ops
 * @author -
 **/

package cs307.first;
import threeohseven.*;


public class BitVectorBig_u83 implements BitVector {
    
    private int low;
    private int high;
    private boolean checked;
    private boolean[] bit_arr;
    private Builder myBuilder;

    public BitVectorBig_u83(int inc_low, int inc_high, boolean inc_checked,
			      Builder inc_builder){
	low = inc_low;
	high = inc_high;
	checked = inc_checked;
	myBuilder = inc_builder;
	int range = high - low + 1; //The number of bits to be stored
	bit_arr = new boolean[range];
	for (int i=0; i<range; i++)
	    bit_arr[i] = false;
    }


    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return value at <code>index</code> 
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index){
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		return bit_arr[true_index];
    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index){
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		boolean old = bit_arr[true_index];
		bit_arr[true_index] = true;
		return old;
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index){
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		boolean old = bit_arr[true_index];
		bit_arr[true_index] = false;
		return old;
    }
    

    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index){
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		boolean old = bit_arr[true_index];
		if (old) bit_arr[true_index] = false;
		else bit_arr[true_index] = true;
		return old;
    }
    

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void and(BitVector bs){
	for (int i=0; i<bit_arr.length; i++)
	    bit_arr[i] = bit_arr[i] && bs.test(i);
    }
   


    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void or(BitVector bs){
	for (int i=0; i<bit_arr.length; i++)
	    bit_arr[i] = bit_arr[i] || bs.test(i);
    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void xor(BitVector bs){
	boolean first;
	boolean second;
	for (int i=0; i<bit_arr.length; i++){
	    first = bit_arr[i];
	    second = bs.test(i);
	    if ( (first&&!second) || (second&&!first) ) bit_arr[i] = true;
	    else bit_arr[i] = false;
	}
    }


    /**
     * Returns a logically disjoint copy of the object.  The operation is
     * only valid if the two <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy(){
	BitVector bv = new BitVectorBig_u83(low, high, checked, myBuilder);
	for (int i =0; i<bit_arr.length; i++){
	    if (bit_arr[i]) bv.set(i);
	    else bv.unset(i);
	}
	return bv;
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    public Iterator iterator(){
	return new Iterator_u83(low, high, this);
    }
    
    /**
     * Returns the builder that created this <code>BitVector</code>.
     **/
    public Builder builder(){
	return myBuilder;
    }

    /**
     * Return a representation of this <code>BitVector</code> as a
     * string. The format is implementation specific.
     **/
    public String toString(){
	String to_return = "Bits in vector: \n";
	for (int i = 0; i<bit_arr.length; i++){
	    to_return = to_return + test(i);
	    if ((i+1) % 8 == 0) to_return = to_return + " ";
	    if ((i+1) % 32 == 0) to_return = to_return + "\n";
	}
	return to_return;
    }


} // end of BitVectorBig_u83

