/**
 * The BitVector_u83 class for 307-PA1, optimized for small-set
 * The bits are bits of individual bytes for easy use of and/or
 * Bits are stored right to left inside each byte
 * @author -
 **/

package cs307.first;
import threeohseven.*;

public class BitVector_u83 implements BitVector {
    
    private int low;
    private int high;
    private boolean checked;
    private byte[] byte_arr;
    private Builder myBuilder;

    public BitVector_u83(int inc_low, int inc_high, boolean inc_checked,
			      Builder inc_builder){
		low = inc_low;
		high = inc_high;
		checked = inc_checked;
		myBuilder = inc_builder;
		int range = high - low + 1; //The number of bits to be stored
		int num_bytes; //Number of bytes needed to store the bits
		if (range % 8 == 0) num_bytes = range / 8;
		else num_bytes = (range / 8) + 1;
		byte_arr = new byte[num_bytes];
		for (int i=0; i<num_bytes; i++)
		    byte_arr[i] = 0;
	}


    /**
     * Test the value at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @author -
     * @param index a value in the active range
     * @return value at <code>index</code> 
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean test(int index){
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		byte bytearea = byte_arr[true_index/8];
		byte bit_pos = (byte)(true_index % 8); //Position right to left of bit
		bytearea = (byte) (bytearea >> bit_pos);
		bytearea = (byte) (bytearea & 0x01);
		return (bytearea == 1);
    }

    /**
     * Set the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @author -
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean set(int index){
		boolean old;
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		byte bytearea = byte_arr[true_index/8];
		byte bit_pos = (byte)(true_index % 8);
		
		//Retrieve old value
		byte temp = (byte)(bytearea >> bit_pos);
		temp = (byte)(temp & 0x01);
		old = (temp == 1);
		
		//Set new value
		temp = (byte)(0x01 << bit_pos);
		byte_arr[true_index/8] = (byte)(bytearea | temp);
		
		return old;
    }

    /**
     * Unset the bit at <code>index</code>.  The operation is only valid if
     * <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @author -
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean unset(int index){
		boolean old;
		int true_index = index-low;
		if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		byte bytearea = byte_arr[true_index/8];
		byte bit_pos = (byte)(true_index % 8);
		int new_mask;
	
		//Retrieve old value
		byte temp = (byte)(bytearea >> bit_pos);
		temp = (byte)(temp & 0x01);
		old = (temp == 1);
		
		//Set new value
		new_mask = 0xFE;
		//Create bitmask with 0 in correct spot
		for (int i=0; i<bit_pos; i++){
		    new_mask = (byte)(new_mask << 1);
		    new_mask = (byte)(new_mask | 0x01);
		}
		//And with old value
		byte_arr[true_index/8] = (byte)(new_mask & bytearea);
	
		return old;
    }
    


    /**
     * Negate the bit at <code>index</code>.  The operation is only valid
     * if <code>index</code> is in the active range of the
     * <code>BitVector</code>.
     * @author -
     * @param index a value in the active range
     * @return old value at <code>index</code>
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public boolean negate(int index){
    	int true_index = index-low;
    	if (true_index <0 || true_index > high){
		    if (checked) throw new ImplementationError("Index out of range");
		    else return false;
		}
		boolean result = test(true_index);
		if (result) unset(true_index);
		else set(true_index);
		return result;
    }
    

    /**
     * Logical <code>and</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if <code>index</code> is in the
     * active range of the <code>BitVector</code>.
     * @author -
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void and(BitVector bs){
	byte newbyte = 0;
	int index;
        boolean result;
	for (int i=0; i<byte_arr.length; i++){
	    for (int j=7; j>=0; j--){
		newbyte = (byte)(newbyte << 1);
		index = i*8+j;
		if (index > (high - low)) continue;
		result = test(index) && bs.test(index);
		if (result) newbyte = (byte) (newbyte | 0x01);
	    }
	    byte_arr[i] = newbyte;
	    newbyte = 0;
	}
    }
   


    /**
     * Logical <code>or</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @author -
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void or(BitVector bs){
	byte newbyte = 0; 
        int index; 
        boolean result; 
        for (int i=0; i<byte_arr.length; i++){ 
            for (int j=7; j>=0; j--){ 
                newbyte = (byte)(newbyte << 1); 
                index = i*8+j; 
                if (index > (high - low)) continue; 
                result = test(index) || bs.test(index); 
                if (result) newbyte = (byte) (newbyte | 0x01); 
            } 
            byte_arr[i] = newbyte; 
            newbyte = 0; 
        } 
    }

    /**
     * Logical <code>xor</code> of the two vectors. Receiver is updated in
     * place.  The operation is only valid if the two
     * <code>BitVector</code> objects are compatible.
     * @param bs the other <code>BitVector</code>.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code> is strict.
     **/
    public void xor(BitVector bs){
	byte newbyte = 0; 
        int index; 
        boolean result; 
        for (int i=0; i<byte_arr.length; i++){ 
            for (int j=7; j>=0; j--){ 
                newbyte = (byte)(newbyte << 1); 
                index = i*8+j; 
                if (index > (high - low)) continue; 
                result = (test(index) && !bs.test(index)) ||
		    (!test(index) && bs.test(index)); 
                if (result) newbyte = (byte) (newbyte | 0x01); 
            } 
            byte_arr[i] = newbyte; 
            newbyte = 0; 
        } 
    }


    /**
     * Returns a logically disjoint copy of the object.  The operation is
     * only valid if the two <code>BitVector</code> objects are compatible.
     * @throws ImplementationError if the operation is invalid and the
     * <code>BitVector</code>
     **/
    public BitVector copy(){
	BitVector_u83 bv = new BitVector_u83(low, high, checked, myBuilder);
	for (int i =0; i<byte_arr.length; i++)
	    bv.setByte(byte_arr[i], i);
	return bv;
    }

    /**
     * Returns an iterator over this <code>BitVector</code>.
     **/
    public Iterator iterator(){
	return new Iterator_u83(low, high, this);
    }
    
    /**
     * Returns the builder that created this <code>BitVector</code>.
     **/
    public Builder builder(){
    	return myBuilder;
    }

    /**
     * Return a representation of this <code>BitVector</code> as a
     * string. The format is implementation specific.
     **/
    public String toString(){
		String to_return = "Bits in vector: \n";
		for (int i = 0; i<byte_arr.length; i++){
		    to_return = to_return + test(i);
		    if ((i+1) % 8 == 0) to_return = to_return + " ";
		    if ((i+1) % 32 == 0) to_return = to_return + "\n";
		}
		return to_return;
    }

    public void setByte(byte toSet, int index){
    	byte_arr[index] = toSet;
    }
       


} // end of BitVector_u83






