/**
 * The Iterator for 307-PA1
 **/
package cs307.first;
import threeohseven.*;


    /**
     * Iterator over a BitVector. <p> An Iterator returns the values of
     * every index in the bit vector's active range in increasing
     * order. The iterator maintains a cursor into the vector, the cursor
     * is guaranteed to be larger or equal to the vector's lower bound.
     *
     * When created an interator points to first element in the active
     * range, so that <code>getNext()</code> returns the first value in the
     * mapping, and <code>position()</code> returns the lower bound of the
     * active range.
     *
     * @author -
     * 
     **/

    public class Iterator_u83 implements BitVector.Iterator {
	
	private int cursor;
	private int high;
	private BitVector bv;


	public Iterator_u83(int inc_low, int inc_high, BitVector inc_bv){
	    cursor = inc_low;
	    high = inc_high;
	    bv = inc_bv;
	}    

	/**
	 * Checks if there are more values in the active range
	 * @return false if the end of the range has been reached.
	 **/
	    public boolean hasNext(){
	    	return (cursor<=high);
	    }

	/**
	 * Returns the value for the current cursor, and advances the
	 * cursor by one. When the cursor overflows the active range
	 * (i.e. <code>hasNext()</code> is false) arbitrary values may be
	 * returned. Implementations are not required to throw an exception.
  	 * @return the value of the next index in range.
  	 * @throws NoSuchElementError if hasNext() == false and vector is strict.
	 **/
	    public boolean getNext(){
			boolean current = bv.test(cursor);
			cursor++;
			return current;
	    }

	/**
	 * The value of the cursor. This value may be larger than the
	 * vector's upper bound. In which case <code>hasNext()</code> will
	 * return false.
	 * @return current position.
	 **/
	    public int position(){
	    	return cursor;
	    }

	/**
	 * Skips to the next set bit such that <code>getNext()</code> will
	 * return true, if none is found the cursor is moved to the end of
	 * the vector and <code>hasNext()</code> will return false from
	 * thereon.
	 * @return position of the first set bit, if there is none an
	 * invalid value will be returned (ie a value larger than the upper
	 * bound of the array).
	 **/
	    public int skipToSet(){
			boolean current;
			while (hasNext() == true){
			    current = bv.test(cursor);
			    if (current) return cursor;
			    cursor++;
			}
			return -1;
	    }

	/**
	 * Skips to the next unset bit such that <code>getNext()</code>
	 * will return false, if none is found the cursor is moved to the
	 * end of the vector and <code>hasNext()</code> will return false
	 * from thereon.
	 * @return position of the first unset bit, if there is none an
	 * invalid value will be returned (ie a value larger than the upper
	 * bound of the array).
	 **/
	    public int skipToUnset(){
			boolean current;
			while (hasNext() == true){
			    current = bv.test(cursor);
			    if (!current) return cursor;
			    cursor++;
			}
			return -1;
	    }

    
} // end of Iterator_u83




