/**
 * Tests for the large version of the bit vector. These tests
 * test the individual vector functions
 * @author -
 */
package test.first.u83;
import cs307.first.*;
import junit.framework.TestCase;
import threeohseven.BitVector;
import threeohseven.ImplementationError;


public class BitVectorBigTest_u83 extends TestCase {

	BitVectorBig_u83 bv;
	
	protected void setUp() throws Exception {
		
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	/**
	 * This function tests that the bits in a vector can be set
	 * and tested
	 * @throws Exception if any operation is illegal since the
	 * vector is strict
	 */
	public void testSet() throws Exception{
		bv = new BitVectorBig_u83(0, 10, true, null);
		bv.set(2);
		assertEquals(true, bv.test(2));
	}
	
	/**
	 * This function tests that bits can be unset in a vector.
	 * @throws Exception if any operation is illegal since the 
	 * vector is strict
	 */
	public void testUnSet() throws Exception{
		bv = new BitVectorBig_u83(0, 10, true, null);
		bv.set(0);
		assertEquals(true, bv.test(0));
		bv.unset(0);
		assertEquals(false, bv.test(0));
	}
	
	/**
	 * This tests the negate function, ensuring that bits can be
	 * set and unset by this function
	 * @throws Exception if any operation on the vector is illegal
	 * since the vector is strict
	 */
	public void testNegate() throws Exception{
		bv = new BitVectorBig_u83(0, 10, true, null);
		bv.set(5);
		assertEquals(true, bv.test(5));
		bv.negate(5);
		assertEquals(false, bv.test(5));
		bv.negate(5);
		assertEquals(true, bv.test(5));
	}
	
	/**
	 * This function tests the copy function of the bitvector.
	 * This is a deep copy. 
	 * @throws Exception if any vector operation is illegal
	 */
	public void testCopy() throws Exception{
		bv = new BitVectorBig_u83(0, 10, true, null);
		bv.set(9);
		assertEquals(true, bv.test(9));
		BitVector cp = bv.copy();
		assertEquals(true, cp.test(9));
	}
	
	
	/**
	 * Tests the logical bitwise and function of the vector.
	 * @throws Exception if any vector operation is illegal
	 */
	public void testAnd() throws Exception{
		bv = new BitVectorBig_u83(0, 3, true, null);
		bv.unset(0);
		bv.unset(1);
		bv.set(2);
		bv.set(3);
		BitVectorBig_u83 bv2 = new BitVectorBig_u83(0, 3, true, null);
		bv2.unset(0);
		bv2.set(1);
		bv2.unset(2);
		bv2.set(3);
		bv.and(bv2);
		assertEquals(false, bv.test(0));
		assertEquals(false, bv.test(1));
		assertEquals(false, bv.test(2));
		assertEquals(true, bv.test(3));
		
	}
	
	/**
	 * Tests the logical bitwise or function of the vector.
	 * @throws Exception if any vector operation is illegal
	 */
	public void testOr() throws Exception{
		bv = new BitVectorBig_u83(0, 3, true, null);
		bv.unset(0);
		bv.unset(1);
		bv.set(2);
		bv.set(3);
		BitVector_u83 bv2 = new BitVector_u83(0, 3, true, null);
		bv2.unset(0);
		bv2.set(1);
		bv2.unset(2);
		bv2.set(3);
		bv.or(bv2);
		assertEquals(false, bv.test(0));
		assertEquals(true, bv.test(1));
		assertEquals(true, bv.test(2));
		assertEquals(true, bv.test(3));
	}
	
	
	/**
	 * Tests the logical bitwise exclusive or function of the vector.
	 * @throws Exception if any vector operation is illegal
	 */
	public void testXor() throws Exception{
		bv = new BitVectorBig_u83(0, 3, true, null);
		bv.unset(0);
		bv.unset(1);
		bv.set(2);
		bv.set(3);
		BitVector_u83 bv2 = new BitVector_u83(0, 3, true, null);
		bv2.unset(0);
		bv2.set(1);
		bv2.unset(2);
		bv2.set(3);
		bv.xor(bv2);
		assertEquals(false, bv.test(0));
		assertEquals(true, bv.test(1));
		assertEquals(true, bv.test(2));
		assertEquals(false, bv.test(3));
	}

	/**
	 * This function tests that errors are correctly thrown
	 * when out of bound elements are attempted to be accessed
	 */
	public void testOutOfBounds(){
		bv = new BitVectorBig_u83(0, 1, true, null);
		try{
			bv.set(2);
			fail();
		}
		catch (ImplementationError e) {
		}	
		
	}

}

