
package test.first.u83;
import junit.framework.TestCase;
import threeohseven.*;
import cs307.first.*;
import threeohseven.Director.Builder; 
import java.util.*;

/**
 * JUnit test for the director class. The director tests that the builder
 * setup is working correctly. It also runs large scale tests to ensure that
 * the different implementations work on a large scale.
 *
 * @author -
 * 
 **/

public class DirectorTest_u83 extends TestCase{
	
	Director_u83 d;
	
	protected void setUp() throws Exception {
		super.setUp();
		d = new Director_u83();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	/**
	 * Test case for the builder pattern. This creates both types of
	 * builders from the director, and ensures that the builders create
	 * working bit vectors.
	 * @throws Exception if any BitVector operation fails and the vector
	 * is strict
	 */
	public void testCreation() throws Exception{
		BitVector.Descriptor desc = new BitVector.Descriptor(0, 10, true, false, true);
		Builder b = d.makeBuilder(desc);
		BitVector bv = (BitVector) b.build();
		bv.set(9);
		assertEquals(true, bv.test(9));
		
		BitVector.Descriptor desc2 = new BitVector.Descriptor(0, 200, false, true, true);
		Builder b2 = d.makeBuilder(desc2);
		BitVector bv2 = (BitVector) b2.build();
		bv2.set(190);
		assertEquals(true, bv2.test(190));
	}
	
	/**
	 * Stress test case for a large number of bit vectors. This creates a very large
	 * number of BitVectors that have been optimized for size. 
	 * @throws Exception if any BitVector operation fails and the vector
	 * is strict
	 */
	public void testSizeStress() throws Exception{
		BitVector.Descriptor desc = new BitVector.Descriptor(0, 127, true, false, true);
		Builder b = d.makeBuilder(desc);
		Vector<BitVector> bvv = new Vector<BitVector>();
		for (int i=0; i<2000; i++)
			bvv.add((BitVector) b.build());
		BitVector bv = bvv.get(1568);
		bv.set(125);
		assertEquals(bv.test(125), true);
		assertEquals(bv.test(124), false);
	}
	
	/**
	 * Stress test case for a bit vector of large sizes. This creates a large BitVector
	 * from the BitVectorBig implementation and runs a large number of set and unset
	 * operations on the vector.
	 * @throws Exception if any BitVector operation fails and the vector
	 * is strict
	 */
	public void testSpeedStress() throws Exception{
		int i;
		BitVector.Descriptor desc = new BitVector.Descriptor(0, 3000, false, true, true);
		Builder b = d.makeBuilder(desc);
		BitVector bv = (BitVector) b.build();
		for (i=0; i<3000; i++)
			if (i%3==0) bv.set(i);
		for (i=0; i<3000; i++)
			if (i%2==0) bv.unset(i);
		assertEquals(bv.test(2100), false);
		assertEquals(bv.test(2103), true);
		assertEquals(bv.test(2999), false);
	}
	
	/**
	 * JUnit test case for the Iterator class. This creates an iterator over a large vector
	 * and tests the operations of the iterator
	 * @throws Exception if Iterator attempts to access an element that does not exist
	 */
	public void testIterator() throws Exception{
		BitVector.Descriptor desc = new BitVector.Descriptor(0, 3000, false, true, true);
		Builder b = d.makeBuilder(desc);
		BitVector bv = (BitVector) b.build();
		bv.set(1500);
		bv.set(3000);
		
		BitVector.Iterator iterator = bv.iterator();
		assertEquals(iterator.hasNext(), true);
		assertEquals(iterator.position(), 0);
		
		iterator.getNext();
		assertEquals(iterator.position(), 1);
		
		iterator.skipToSet();
		assertEquals(iterator.position(), 1500);
		assertEquals(iterator.getNext(), true);
		
		iterator.skipToSet();
		assertEquals(iterator.position(), 3000);
		
		assertEquals(iterator.getNext(), true);
		assertEquals(iterator.hasNext(), false);
	
	}
	
}
