package cs307.first;
import java.util.Vector;

public class BitVector_93 implements BitVector {

	private int low;
	private int high;
	private Iterator itter;
	private Builder b;
	private Descriptor dis;
	private Vector<Boolean> data;
	
	/**
	 * ======================= Constructors =======================
	 **/
	
	 public BitVector_93()
	 {
		low = 0;
		high = 1;
		itter = null;
		b = null;
		dis = null;
		data = new Vector<Boolean>();
		data.add(false);
	 }
	 
	 public BitVector_93(Descriptor dd)
	 {
		low = dd.getLowerBound();
		high = dd.getUpperBound();
		dis = dd;
		b = null;
		//itter = new Iterator(low, high);
		data = new Vector<Boolean>();
		for(int i = 0; i < high; i++) //set all values of data to false
			data.add(false);
	 }
	 
	/**
	 * ==================== General Methods =====================
	 **/

	 /******
	 * Name: and
		 * 
		 * Function: Logical and of two BitVectors
		 * 
		 * Params: BitVector bs - the other BitVector to be anded
		 * 
		 * Throws: ImplementationError if not allowed and current BitVector is strict.
	    ******/
	public void and(BitVector bs) {//throws ImplementationError{
		if(this.builder().compatible(bs.builder()))  //If the builders are compatible
		{
			int i = this.low;
			while(i < high) {
				if(test(i) == bs.test(i))  //if they are equal
					set(i);
				else 
					unset(i);
				i++;
			}
		}
		else if(this.dis.boundCheckRequired())
			try {
				throw new ImplementationError("and.  Builders not compatible");
			} catch (ImplementationError e) {
				System.out.println(e.toString());
				e.printStackTrace();
			}
	}

	/**
	 * Name: builder()
	 * Function: Returns the ID code of the BitVector's Builder
	 */
	public Builder builder() {
		return b;
	}

	/**
	 * Returns a logically disjoint, deep copy of this BitVector
	 */
	public BitVector copy() {
		Builder bld = this.builder();
		return bld.buildBitVector(this);
	}

	/**
	 * Returns the iterator of the BitVector (always returns null)
	 */
	public Iterator iterator() {
		return itter;
	}

	/******
	 * Name: negate
	 * 
	 * Function: inverts the value of the bit at location 'index'
	 * 
	 * Params: int index - the location of the bit to be inverted
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public boolean negate(int index) {
		if(test(index)) {
			data.set(index, false);
			return true;
		} else
			data.set(index,true);
		return false;
	}

	/******
	 * Name: or
	 * 
	 * Function: Logical operation or of two BitVectors
	 * 
	 * Params: BitVector bs - the other BitVector to or with the current BitVector
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public void or(BitVector bs) {
		if(this.builder().compatible(bs.builder()))  //If the builders are compatible
		{
			int i = this.low;
			while(i < high) {
				if(test(i) == false && bs.test(i) == false)  //if they are both false
					unset(i);
				else 
					set(i);
				i++;
			}	
		}
		else if(this.dis.boundCheckRequired())
			try {
				throw new ImplementationError("Or.  Builders not compatible");
			} catch (ImplementationError e) {
				System.out.println(e.toString());
				e.printStackTrace();
			}
	}

	/******
	 * Name: set
	 * 
	 * Function: Sets the value of the bit at location 'index' to true
	 * 
	 * Params: int index - the location of the bit to be set
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public boolean set(int index) {
		if(test(index)) 
			return true;
		else 
			data.set(index, true);
		return false;
	}

	/******
	 * Name: test
	 * 
	 * Function: Returns the value of the bit at location 'index'
	 * 
	 * Params: int index - the location of the bit to be tested
	 * 
	 * Returns: The boolean value of the bit at location 'index'
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public boolean test(int index) {
		//check if index is between low and high
		if(index >= low && index < high)
			return data.get(index);
		else if(dis.boundCheckRequired())
			try {
				throw new ImplementationError("Test.  Index out of bounds");
			} catch (ImplementationError e) {
				System.out.println(e.toString());
				e.printStackTrace();
			}
		return false;
	}

	/******
	 * Name: unset
	 * 
	 * Function: Sets the bit at 'index' to false
	 * 
	 * Params: int index - the location in the bit in the vector that is to be unset
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public boolean unset(int index) {
		if(!test(index)) 
			return false;
		else 
			data.set(index, false);
		return true;
	}

	/******
	 * Name: xor
	 * 
	 * Function: Logical xor of two BitVectors.  The result of this operation
	 * 		overwrites the current BitVector.
	 * 
	 * Params: BitVector bs - the other BitVetor to be Xored along with the
	 * 						  current BitVector.  This does not change bs.
	 * 
	 * Throws: ImplementationError if not allowed and current BitVector is strict.
    ******/
	public void xor(BitVector bs) {
		if(this.builder().compatible(bs.builder()))  //If the builders are compatible
		{
			int i = low;
			while (i < high) {
				if(test(i) != bs.test(i))   //if they aren't equal
					set(i);
				else 
					unset(i);
				i++;
			}
		}
		else if(this.dis.boundCheckRequired())
			try {
				throw new ImplementationError("Xor.  Builders not compatible");
			} catch (ImplementationError e) {
				System.out.println(e.toString());
				e.printStackTrace();
			}
	}

	 // ========================= Get/Set Methods ==========================
	 
	/**
	 * Returns the Lower Bound of the BitVector
	 */
	public int getLow()
	{
		return low;
	}
	/**
	 * Returns the Upper Bound of the BitVector
	 */
	public int getHigh()
	{
		return high;
	}
	/**
	 * Returns the Vector portion of the BitVector (i.e the data)
	 */
	public Vector<Boolean> getData()
	{
		return data;
	}
	/**
	 * Returns the Descriptor of the BitVector
	 */
	public Descriptor getDis(){
		return dis;
	}
	/**
	 * Function: A simple set method that allows one to manually set the 
	 * 			builder pointer of the BitVector
	 * 			(NOTE: This could be used to falsely point the BitVector to 
	 * 					the wrong builder)
	 * Param: Bulder_bdharcou a - The builder that is to be set as the BitVector's builder
	 */
	public void setBuilder(Builder_93 a){
		b = a;
	}

	public static final class ImplementationError extends Exception{

		String msg;
		public ImplementationError(){
			msg = "Unknown Error has occured";
		}
		
		public ImplementationError(String a){
			msg = a;
		}
		
		public String toString(){
			return ("WARNING: error occured in process " + msg);
		}
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		
		
	}
}
